@@ -1,6 +1,7 @@
 package incus
 
 import (
+	"errors"
 	"fmt"
 	"net/url"
 
@@ -36,6 +37,23 @@ func (r *ProtocolIncus) GetCertificates() ([]api.Certificate, error) {
 	return certificates, nil
 }
 
+// GetCertificatesWithFilter returns a filtered list of certificates.
+func (r *ProtocolIncus) GetCertificatesWithFilter(filters []string) ([]api.Certificate, error) {
+	certificates := []api.Certificate{}
+
+	v := url.Values{}
+	v.Set("recursion", "1")
+	v.Set("filter", parseFilters(filters))
+
+	// Fetch the raw value
+	_, err := r.queryStruct("GET", fmt.Sprintf("/certificates?%s", v.Encode()), nil, "", &certificates)
+	if err != nil {
+		return nil, err
+	}
+
+	return certificates, nil
+}
+
 // GetCertificate returns the certificate entry for the provided fingerprint.
 func (r *ProtocolIncus) GetCertificate(fingerprint string) (*api.Certificate, string, error) {
 	certificate := api.Certificate{}
@@ -63,7 +81,7 @@ func (r *ProtocolIncus) CreateCertificate(certificate api.CertificatesPost) erro
 // UpdateCertificate updates the certificate definition.
 func (r *ProtocolIncus) UpdateCertificate(fingerprint string, certificate api.CertificatePut, ETag string) error {
 	if !r.HasExtension("certificate_update") {
-		return fmt.Errorf("The server is missing the required \"certificate_update\" API extension")
+		return errors.New("The server is missing the required \"certificate_update\" API extension")
 	}
 
 	// Send the request
@@ -89,11 +107,11 @@ func (r *ProtocolIncus) DeleteCertificate(fingerprint string) error {
 // CreateCertificateToken requests a certificate add token.
 func (r *ProtocolIncus) CreateCertificateToken(certificate api.CertificatesPost) (Operation, error) {
 	if !r.HasExtension("certificate_token") {
-		return nil, fmt.Errorf("The server is missing the required \"certificate_token\" API extension")
+		return nil, errors.New("The server is missing the required \"certificate_token\" API extension")
 	}
 
 	if !certificate.Token {
-		return nil, fmt.Errorf("Token needs to be true if requesting a token")
+		return nil, errors.New("Token needs to be true if requesting a token")
 	}
 
 	// Send the request
