@@ -414,7 +414,7 @@ func (f *Field) OrderBy(mapping *Mapping, primaryTable string) (string, error) {
 
 // JoinClause returns an SQL 'JOIN' clause using the 'join'  and 'joinon' tags, if present.
 func (f *Field) JoinClause(mapping *Mapping, table string) (string, error) {
-	joinTemplate := "\n  JOIN %s ON %s = %s.id"
+	joinTemplate := "\n  JOIN %s ON %s = %s.%s"
 	if f.Config.Get("join") != "" && f.Config.Get("leftjoin") != "" {
 		return "", fmt.Errorf("Cannot join and leftjoin at the same time for field %q of struct %q", f.Name, mapping.Name)
 	}
@@ -448,7 +448,12 @@ func (f *Field) JoinClause(mapping *Mapping, table string) (string, error) {
 		return "", fmt.Errorf("'joinon' tag of field %q of struct %q must be of form '<table>.<column>'", f.Name, mapping.Name)
 	}
 
-	return fmt.Sprintf(joinTemplate, joinTable, joinOn, joinTable), nil
+	joinTo := "id"
+	if f.Config.Get("jointo") != "" {
+		joinTo = f.Config.Get("jointo")
+	}
+
+	return fmt.Sprintf(joinTemplate, joinTable, joinOn, joinTable, joinTo), nil
 }
 
 // InsertColumn returns a column name and parameter value suitable for an 'INSERT', 'UPDATE', or 'DELETE' statement.
@@ -456,7 +461,7 @@ func (f *Field) JoinClause(mapping *Mapping, table string) (string, error) {
 // to select the ID to insert into this table.
 // - If a 'joinon' tag is present, but this table is not among the conditions, then the join will be considered indirect,
 // and an empty string will be returned.
-func (f *Field) InsertColumn(mapping *Mapping, primaryTable string, defs map[*ast.Ident]types.Object, registeredSQLStmts map[string]string) (string, string, error) {
+func (f *Field) InsertColumn(mapping *Mapping, primaryTable string, defs map[*ast.Ident]types.Object, registeredSQLStmts map[string]string, allFields []*Field) (string, string, error) {
 	var column string
 	var value string
 	var err error
@@ -501,6 +506,37 @@ func (f *Field) InsertColumn(mapping *Mapping, primaryTable string, defs map[*as
 			return "", "", fmt.Errorf("Failed to find registered statement %q for field %q of struct %q: %w", varName, f.Name, mapping.Name, err)
 		}
 
+		// Keep track of the join config of other fields that have a corresponding table column.
+		otherJoins := map[string]string{}
+		for _, otherField := range allFields {
+			if f.Name == otherField.Name {
+				continue
+			}
+
+			joinCfg := otherField.joinConfig()
+			table, _, ok := strings.Cut(joinCfg, ".")
+			if !ok {
+				continue
+			}
+
+			// If 'joinon' points to a different table, then there is no column on the table for this field.
+			joinOn := otherField.Config.Get("joinon")
+			if joinOn == "" || strings.HasPrefix(joinOn, tableName+".") {
+				otherJoins[table] = joinCfg
+			}
+		}
+
+		// If the field maps to a column with a foreign key to table A, but table A has a composite key with table B,
+		// then if we already have a field mapping to table B, just reuse its ID.
+		if strings.Contains(joinStmt, "JOIN ") && strings.Contains(joinStmt, "WHERE ") {
+			wheres := strings.Split(joinStmt, "WHERE ")
+			for table, joinCfg := range otherJoins {
+				wheres[1] = strings.ReplaceAll(wheres[1], joinCfg+" = ?", table+".id = "+lex.Singular(table)+"_id")
+			}
+
+			joinStmt = strings.Join(wheres, "WHERE ")
+		}
+
 		value = fmt.Sprintf("(%s)", strings.ReplaceAll(strings.ReplaceAll(joinStmt, "`", ""), "\n", ""))
 		value = strings.ReplaceAll(value, "  ", " ")
 	} else {
