@@ -7,33 +7,32 @@ import (
 	"fmt"
 	"io"
 	"io/fs"
+	"maps"
 	"net/http"
 	"os"
 	"os/exec"
 	"strconv"
 	"sync"
-	"syscall"
 	"time"
 
 	"github.com/gorilla/websocket"
-	"golang.org/x/sys/unix"
 
 	"github.com/lxc/incus/v6/internal/jmap"
-	"github.com/lxc/incus/v6/internal/linux"
 	"github.com/lxc/incus/v6/internal/server/db/operationtype"
 	"github.com/lxc/incus/v6/internal/server/operations"
 	"github.com/lxc/incus/v6/internal/server/response"
 	internalUtil "github.com/lxc/incus/v6/internal/util"
 	"github.com/lxc/incus/v6/shared/api"
 	"github.com/lxc/incus/v6/shared/logger"
-	"github.com/lxc/incus/v6/shared/util"
 	"github.com/lxc/incus/v6/shared/ws"
 )
 
-const execWSControl = -1
-const execWSStdin = 0
-const execWSStdout = 1
-const execWSStderr = 2
+const (
+	execWSControl = -1
+	execWSStdin   = 0
+	execWSStdout  = 1
+	execWSStderr  = 2
+)
 
 var execCmd = APIEndpoint{
 	Name: "exec",
@@ -43,6 +42,10 @@ var execCmd = APIEndpoint{
 }
 
 func execPost(d *Daemon, r *http.Request) response.Response {
+	if d.Features != nil && !d.Features["exec"] {
+		return response.Forbidden(errors.New("Command execution has been disabled by configuration"))
+	}
+
 	post := api.InstanceExecPost{}
 
 	buf, err := io.ReadAll(r.Body)
@@ -56,90 +59,51 @@ func execPost(d *Daemon, r *http.Request) response.Response {
 	}
 
 	if !post.WaitForWS {
-		return response.BadRequest(fmt.Errorf("Websockets are required for VM exec"))
+		return response.BadRequest(errors.New("Websockets are required for VM exec"))
 	}
 
 	env := map[string]string{}
 
 	if post.Environment != nil {
-		for k, v := range post.Environment {
-			env[k] = v
-		}
+		maps.Copy(env, post.Environment)
 	}
 
-	// Set default value for PATH
-	_, ok := env["PATH"]
-	if !ok {
-		env["PATH"] = "/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin"
-	}
+	osSetEnv(&post, env)
 
-	if util.PathExists("/snap/bin") {
-		env["PATH"] = fmt.Sprintf("%s:/snap/bin", env["PATH"])
-	}
+	webSocket := &execWs{}
+	webSocket.fds = map[int]string{}
 
-	// If running as root, set some env variables
-	if post.User == 0 {
-		// Set default value for HOME
-		_, ok = env["HOME"]
-		if !ok {
-			env["HOME"] = "/root"
-		}
-
-		// Set default value for USER
-		_, ok = env["USER"]
-		if !ok {
-			env["USER"] = "root"
-		}
-	}
-
-	// Set default value for LANG
-	_, ok = env["LANG"]
-	if !ok {
-		env["LANG"] = "C.UTF-8"
-	}
-
-	// Set the default working directory
-	if post.Cwd == "" {
-		post.Cwd = env["HOME"]
-		if post.Cwd == "" {
-			post.Cwd = "/"
-		}
-	}
-
-	ws := &execWs{}
-	ws.fds = map[int]string{}
-
-	ws.conns = map[int]*websocket.Conn{}
-	ws.conns[execWSControl] = nil
-	ws.conns[0] = nil // This is used for either TTY or Stdin.
+	webSocket.conns = map[int]*websocket.Conn{}
+	webSocket.conns[execWSControl] = nil
+	webSocket.conns[0] = nil // This is used for either TTY or Stdin.
 	if !post.Interactive {
-		ws.conns[execWSStdout] = nil
-		ws.conns[execWSStderr] = nil
+		webSocket.conns[execWSStdout] = nil
+		webSocket.conns[execWSStderr] = nil
 	}
 
-	ws.requiredConnectedCtx, ws.requiredConnectedDone = context.WithCancel(context.Background())
-	ws.interactive = post.Interactive
+	webSocket.requiredConnectedCtx, webSocket.requiredConnectedDone = context.WithCancel(context.Background())
+	webSocket.interactive = post.Interactive
 
-	for i := range ws.conns {
-		ws.fds[i], err = internalUtil.RandomHexString(32)
+	for i := range webSocket.conns {
+		webSocket.fds[i], err = internalUtil.RandomHexString(32)
 		if err != nil {
 			return response.InternalError(err)
 		}
 	}
 
-	ws.command = post.Command
-	ws.env = env
+	webSocket.command = post.Command
+	webSocket.env = env
 
-	ws.width = post.Width
-	ws.height = post.Height
+	webSocket.width = post.Width
+	webSocket.height = post.Height
 
-	ws.cwd = post.Cwd
-	ws.uid = post.User
-	ws.gid = post.Group
+	webSocket.cwd = post.Cwd
+	webSocket.uid = post.User
+	webSocket.gid = post.Group
 
 	resources := map[string][]api.URL{}
 
-	op, err := operations.OperationCreate(nil, "", operations.OperationClassWebsocket, operationtype.CommandExec, resources, ws.Metadata(), ws.Do, nil, ws.Connect, r)
+	op, err := operations.OperationCreate(nil, "", operations.OperationClassWebsocket, operationtype.CommandExec, resources, webSocket.Metadata(), webSocket.Do, nil, webSocket.Connect, r)
 	if err != nil {
 		return response.InternalError(err)
 	}
@@ -187,7 +151,7 @@ func (s *execWs) Metadata() any {
 func (s *execWs) Connect(op *operations.Operation, r *http.Request, w http.ResponseWriter) error {
 	secret := r.FormValue("secret")
 	if secret == "" {
-		return fmt.Errorf("missing secret")
+		return errors.New("missing secret")
 	}
 
 	for fd, fdSecret := range s.fds {
@@ -198,7 +162,7 @@ func (s *execWs) Connect(op *operations.Operation, r *http.Request, w http.Respo
 			}
 
 			s.connsLock.Lock()
-			defer s.connsLock.Unlock()
+			defer s.connsLock.Unlock() //nolint:revive
 
 			val, found := s.conns[fd]
 			if found && val == nil {
@@ -213,10 +177,10 @@ func (s *execWs) Connect(op *operations.Operation, r *http.Request, w http.Respo
 				s.requiredConnectedDone() // All required connections now connected.
 				return nil
 			} else if !found {
-				return fmt.Errorf("Unknown websocket number")
-			} else {
-				return fmt.Errorf("Websocket number already connected")
+				return errors.New("Unknown websocket number")
 			}
+
+			return errors.New("Websocket number already connected")
 		}
 	}
 
@@ -242,23 +206,25 @@ func (s *execWs) Do(op *operations.Operation) error {
 	logger.Debug("Waiting for exec websockets to connect")
 	select {
 	case <-s.requiredConnectedCtx.Done():
+		//nolint:revive //whyNoLint: this is intentional, the flow should continue if all websockets are connected
 		break
 	case <-time.After(time.Second * 5):
-		return fmt.Errorf("Timed out waiting for websockets to connect")
+		return errors.New("Timed out waiting for websockets to connect")
 	}
 
 	var err error
-	var ttys []*os.File
-	var ptys []*os.File
+	var ttys []io.ReadWriteCloser
+	var ptys []io.ReadWriteCloser
 
-	var stdin *os.File
-	var stdout *os.File
-	var stderr *os.File
+	var stdin io.ReadCloser
+	var stdout io.WriteCloser
+	var stderr io.WriteCloser
 
 	if s.interactive {
-		ttys = make([]*os.File, 1)
-		ptys = make([]*os.File, 1)
-		ptys[0], ttys[0], err = linux.OpenPty(int64(s.uid), int64(s.gid))
+		ttys = make([]io.ReadWriteCloser, 1)
+		ptys = make([]io.ReadWriteCloser, 1)
+
+		ptys[0], ttys[0], err = osGetInteractiveConsole(s)
 		if err != nil {
 			return err
 		}
@@ -266,14 +232,10 @@ func (s *execWs) Do(op *operations.Operation) error {
 		stdin = ttys[0]
 		stdout = ttys[0]
 		stderr = ttys[0]
-
-		if s.width > 0 && s.height > 0 {
-			_ = linux.SetPtySize(int(ptys[0].Fd()), s.width, s.height)
-		}
 	} else {
-		ttys = make([]*os.File, 3)
-		ptys = make([]*os.File, 3)
-		for i := 0; i < len(ttys); i++ {
+		ttys = make([]io.ReadWriteCloser, 3)
+		ptys = make([]io.ReadWriteCloser, 3)
+		for i := range ttys {
 			ptys[i], ttys[i], err = os.Pipe()
 			if err != nil {
 				return err
@@ -285,10 +247,14 @@ func (s *execWs) Do(op *operations.Operation) error {
 		stderr = ttys[execWSStderr]
 	}
 
+	ctxCommand, cancel := context.WithCancel(context.Background())
 	waitAttachedChildIsDead, markAttachedChildIsDead := context.WithCancel(context.Background())
 	var wgEOF sync.WaitGroup
 
 	finisher := func(cmdResult int, cmdErr error) error {
+		// Cancel the context after we're done with cleanup.
+		defer cancel()
+
 		// Cancel this before closing the control connection so control handler can detect command ending.
 		markAttachedChildIsDead()
 
@@ -322,9 +288,9 @@ func (s *execWs) Do(op *operations.Operation) error {
 	var cmd *exec.Cmd
 
 	if len(s.command) > 1 {
-		cmd = exec.Command(s.command[0], s.command[1:]...)
+		cmd = exec.CommandContext(ctxCommand, s.command[0], s.command[1:]...)
 	} else {
-		cmd = exec.Command(s.command[0])
+		cmd = exec.CommandContext(ctxCommand, s.command[0])
 	}
 
 	// Prepare the environment
@@ -335,32 +301,15 @@ func (s *execWs) Do(op *operations.Operation) error {
 	cmd.Stdin = stdin
 	cmd.Stdout = stdout
 	cmd.Stderr = stderr
-	cmd.SysProcAttr = &syscall.SysProcAttr{
-		Credential: &syscall.Credential{
-			Uid: s.uid,
-			Gid: s.gid,
-		},
-		// Creates a new session if the calling process is not a process group leader.
-		// The calling process is the leader of the new session, the process group leader of
-		// the new process group, and has no controlling terminal.
-		// This is important to allow remote shells to handle ctrl+c.
-		Setsid: true,
-	}
-
-	// Make the given terminal the controlling terminal of the calling process.
-	// The calling process must be a session leader and not have a controlling terminal already.
-	// This is important as allows ctrl+c to work as expected for non-shell programs.
-	if s.interactive {
-		cmd.SysProcAttr.Setctty = true
-	}
-
 	cmd.Dir = s.cwd
 
+	osPrepareExecCommand(s, cmd)
+
 	err = cmd.Start()
 	if err != nil {
 		exitStatus := -1
 
-		if errors.Is(err, exec.ErrNotFound) || os.IsNotExist(err) {
+		if errors.Is(err, exec.ErrNotFound) || errors.Is(err, fs.ErrNotExist) {
 			exitStatus = 127
 		} else if errors.Is(err, fs.ErrPermission) {
 			exitStatus = 126
@@ -397,12 +346,7 @@ func (s *execWs) Do(op *operations.Operation) error {
 					l.Warn("Failed getting exec control websocket reader, killing command", logger.Ctx{"err": err})
 				}
 
-				err := unix.Kill(cmd.Process.Pid, unix.SIGKILL)
-				if err != nil {
-					l.Error("Failed to send SIGKILL")
-				} else {
-					l.Info("Sent SIGKILL")
-				}
+				cancel()
 
 				return
 			}
@@ -419,40 +363,14 @@ func (s *execWs) Do(op *operations.Operation) error {
 				return
 			}
 
-			command := api.InstanceExecControl{}
-			err = json.Unmarshal(buf, &command)
+			control := api.InstanceExecControl{}
+			err = json.Unmarshal(buf, &control)
 			if err != nil {
 				l.Debug("Failed to unmarshal control socket command", logger.Ctx{"err": err})
 				continue
 			}
 
-			if command.Command == "window-resize" && s.interactive {
-				winchWidth, err := strconv.Atoi(command.Args["width"])
-				if err != nil {
-					l.Debug("Unable to extract window width", logger.Ctx{"err": err})
-					continue
-				}
-
-				winchHeight, err := strconv.Atoi(command.Args["height"])
-				if err != nil {
-					l.Debug("Unable to extract window height", logger.Ctx{"err": err})
-					continue
-				}
-
-				err = linux.SetPtySize(int(ptys[0].Fd()), winchWidth, winchHeight)
-				if err != nil {
-					l.Debug("Failed to set window size", logger.Ctx{"err": err, "width": winchWidth, "height": winchHeight})
-					continue
-				}
-			} else if command.Command == "signal" {
-				err := unix.Kill(cmd.Process.Pid, unix.Signal(command.Signal))
-				if err != nil {
-					l.Debug("Failed forwarding signal", logger.Ctx{"err": err, "signal": command.Signal})
-					continue
-				}
-
-				l.Info("Forwarded signal", logger.Ctx{"signal": command.Signal})
-			}
+			osHandleExecControl(control, s, ptys[0], cmd, l)
 		}
 	}()
 
@@ -468,7 +386,7 @@ func (s *execWs) Do(op *operations.Operation) error {
 			conn := s.conns[0]
 			s.connsLock.Unlock()
 
-			readDone, writeDone := ws.Mirror(conn, linux.NewExecWrapper(waitAttachedChildIsDead, ptys[0]))
+			readDone, writeDone := ws.Mirror(conn, osExecWrapper(waitAttachedChildIsDead, ptys[0]))
 
 			<-readDone
 			<-writeDone
@@ -476,7 +394,7 @@ func (s *execWs) Do(op *operations.Operation) error {
 		}()
 	} else {
 		wgEOF.Add(len(ttys) - 1)
-		for i := 0; i < len(ttys); i++ {
+		for i := range ttys {
 			go func(i int) {
 				l.Debug("Exec mirror websocket started", logger.Ctx{"number": i})
 				defer l.Debug("Exec mirror websocket finished", logger.Ctx{"number": i})
@@ -501,7 +419,7 @@ func (s *execWs) Do(op *operations.Operation) error {
 		}
 	}
 
-	exitStatus, err := linux.ExitStatus(cmd.Wait())
+	exitStatus, err := osExitStatus(cmd.Wait())
 
 	l.Debug("Instance process stopped", logger.Ctx{"err": err, "exitStatus": exitStatus})
 	return finisher(exitStatus, nil)
