@@ -1,7 +1,9 @@
 package main
 
 import (
+	"fmt"
 	"os"
+	"runtime"
 
 	"github.com/spf13/cobra"
 
@@ -9,8 +11,10 @@ import (
 )
 
 type cmdGlobal struct {
-	flagVersion bool
-	flagHelp    bool
+	flagVersion         bool
+	flagHelp            bool
+	flagService         bool
+	flagSecretsLocation string
 
 	flagLogVerbose bool
 	flagLogDebug   bool
@@ -33,6 +37,10 @@ func main() {
 	app.PersistentFlags().BoolVarP(&globalCmd.flagHelp, "help", "h", false, "Print help")
 	app.PersistentFlags().BoolVarP(&globalCmd.flagLogVerbose, "verbose", "v", false, "Show all information messages")
 	app.PersistentFlags().BoolVarP(&globalCmd.flagLogDebug, "debug", "d", false, "Show all debug messages")
+	app.PersistentFlags().StringVarP(&globalCmd.flagSecretsLocation, "secrets-location", "s", "", "Secrets location of the certificate and private key")
+	if runtime.GOOS == "windows" {
+		app.PersistentFlags().BoolVar(&globalCmd.flagService, "service", false, "Start as a system service")
+	}
 
 	// Version handling
 	app.SetVersionTemplate("{{.Version}}\n")
@@ -41,6 +49,7 @@ func main() {
 	// Run the main command and handle errors
 	err := app.Execute()
 	if err != nil {
+		fmt.Fprintf(os.Stderr, "Error: %v\n", err)
 		os.Exit(1)
 	}
 }
