@@ -3,7 +3,7 @@ package main
 import (
 	"net/http"
 
-	"github.com/lxc/incus/shared/api"
+	"github.com/lxc/incus/v6/shared/api"
 )
 
 type devIncusResponse struct {
