@@ -3,11 +3,9 @@
 package main
 
 import (
-	"encoding/pem"
+	"errors"
 	"fmt"
 	"net"
-	"net/http"
-	"os"
 	"os/exec"
 	"slices"
 	"strconv"
@@ -22,55 +20,46 @@ import (
 	"github.com/lxc/incus/v6/internal/linux"
 	"github.com/lxc/incus/v6/internal/ports"
 	internalUtil "github.com/lxc/incus/v6/internal/util"
-	"github.com/lxc/incus/v6/internal/version"
 	"github.com/lxc/incus/v6/shared/api"
 	"github.com/lxc/incus/v6/shared/subprocess"
-	localtls "github.com/lxc/incus/v6/shared/tls"
 	"github.com/lxc/incus/v6/shared/util"
 	"github.com/lxc/incus/v6/shared/validate"
 )
 
-func (c *cmdAdminInit) RunInteractive(cmd *cobra.Command, args []string, d incus.InstanceServer, server *api.Server) (*api.InitPreseed, error) {
+// RunInteractive runs the actual command logic.
+func (c *cmdAdminInit) RunInteractive(_ *cobra.Command, d incus.InstanceServer, server *api.Server) (*api.InitPreseed, error) {
 	// Initialize config
-	config := api.InitPreseed{}
-	config.Server.Config = map[string]string{}
-	config.Server.Networks = []api.InitNetworksProjectPost{}
-	config.Server.StoragePools = []api.StoragePoolsPost{}
-	config.Server.Profiles = []api.InitProfileProjectPost{
-		{
-			ProfilesPost: api.ProfilesPost{
-				Name: "default",
-				ProfilePut: api.ProfilePut{
-					Config:  map[string]string{},
-					Devices: map[string]map[string]string{},
-				},
-			},
-			Project: api.ProjectDefaultName,
-		},
-	}
+	config := NewInitPressed()
 
 	// Clustering
-	err := c.askClustering(&config, d, server)
+	clustering, err := c.global.asker.AskBool(i18n.G("Would you like to use clustering?")+" (yes/no) [default=no]: ", "no")
 	if err != nil {
 		return nil, err
 	}
 
+	if clustering {
+		err := askClustering(c.global.asker, config, nil, d, false)
+		if err != nil {
+			return nil, err
+		}
+	}
+
 	// Ask all the other questions
 	if config.Cluster == nil || config.Cluster.ClusterAddress == "" {
 		// Storage
-		err = c.askStorage(&config, d, server)
+		err = c.askStorage(config, d, server)
 		if err != nil {
 			return nil, err
 		}
 
 		// Networking
-		err = c.askNetworking(&config, d)
+		err = c.askNetworking(config, d)
 		if err != nil {
 			return nil, err
 		}
 
 		// Daemon config
-		err = c.askDaemon(&config, d, server)
+		err = c.askDaemon(config, server)
 		if err != nil {
 			return nil, err
 		}
@@ -92,7 +81,7 @@ func (c *cmdAdminInit) RunInteractive(cmd *cobra.Command, args []string, d incus
 			object = api.InitPreseed{}
 			object.Cluster = config.Cluster
 		} else {
-			object = config
+			object = *config
 		}
 
 		out, err := yaml.Marshal(object)
@@ -103,192 +92,7 @@ func (c *cmdAdminInit) RunInteractive(cmd *cobra.Command, args []string, d incus
 		fmt.Printf("%s\n", out)
 	}
 
-	return &config, nil
-}
-
-func (c *cmdAdminInit) askClustering(config *api.InitPreseed, d incus.InstanceServer, server *api.Server) error {
-	clustering, err := c.global.asker.AskBool(i18n.G("Would you like to use clustering?")+" (yes/no) [default=no]: ", "no")
-	if err != nil {
-		return err
-	}
-
-	if clustering {
-		config.Cluster = &api.InitClusterPreseed{}
-		config.Cluster.Enabled = true
-
-		askForServerName := func() error {
-			config.Cluster.ServerName, err = c.global.asker.AskString(fmt.Sprintf(i18n.G("What member name should be used to identify this server in the cluster?")+" [default=%s]: ", c.defaultHostname()), c.defaultHostname(), nil)
-			if err != nil {
-				return err
-			}
-
-			return nil
-		}
-
-		// Cluster server address
-		address := internalUtil.NetworkInterfaceAddress()
-		validateServerAddress := func(value string) error {
-			address := internalUtil.CanonicalNetworkAddress(value, ports.HTTPSDefaultPort)
-
-			host, _, _ := net.SplitHostPort(address)
-			if slices.Contains([]string{"", "[::]", "0.0.0.0"}, host) {
-				return fmt.Errorf(i18n.G("Invalid IP address or DNS name"))
-			}
-
-			if err == nil {
-				if server.Config["cluster.https_address"] == address || server.Config["core.https_address"] == address {
-					// We already own the address, just move on.
-					return nil
-				}
-			}
-
-			listener, err := net.Listen("tcp", address)
-			if err != nil {
-				return fmt.Errorf(i18n.G("Can't bind address %q: %w"), address, err)
-			}
-
-			_ = listener.Close()
-			return nil
-		}
-
-		serverAddress, err := c.global.asker.AskString(fmt.Sprintf(i18n.G("What IP address or DNS name should be used to reach this server?")+" [default=%s]: ", address), address, validateServerAddress)
-		if err != nil {
-			return err
-		}
-
-		serverAddress = internalUtil.CanonicalNetworkAddress(serverAddress, ports.HTTPSDefaultPort)
-		config.Server.Config["core.https_address"] = serverAddress
-
-		clusterJoin, err := c.global.asker.AskBool(i18n.G("Are you joining an existing cluster?")+" (yes/no) [default=no]: ", "no")
-		if err != nil {
-			return err
-		}
-
-		if clusterJoin {
-			// Existing cluster
-			config.Cluster.ServerAddress = serverAddress
-
-			// Root is required to access the certificate files
-			if os.Geteuid() != 0 {
-				return fmt.Errorf(i18n.G("Joining an existing cluster requires root privileges"))
-			}
-
-			var joinToken *api.ClusterMemberJoinToken
-
-			validJoinToken := func(input string) error {
-				j, err := internalUtil.JoinTokenDecode(input)
-				if err != nil {
-					return fmt.Errorf(i18n.G("Invalid join token: %w"), err)
-				}
-
-				joinToken = j // Store valid decoded join token
-				return nil
-			}
-
-			clusterJoinToken, err := c.global.asker.AskString(i18n.G("Please provide join token:")+" ", "", validJoinToken)
-			if err != nil {
-				return err
-			}
-
-			// Set server name from join token
-			config.Cluster.ServerName = joinToken.ServerName
-
-			// Attempt to find a working cluster member to use for joining by retrieving the
-			// cluster certificate from each address in the join token until we succeed.
-			for _, clusterAddress := range joinToken.Addresses {
-				config.Cluster.ClusterAddress = internalUtil.CanonicalNetworkAddress(clusterAddress, ports.HTTPSDefaultPort)
-
-				// Cluster certificate
-				cert, err := localtls.GetRemoteCertificate(fmt.Sprintf("https://%s", config.Cluster.ClusterAddress), version.UserAgent)
-				if err != nil {
-					fmt.Printf(i18n.G("Error connecting to existing cluster member %q: %v")+"\n", clusterAddress, err)
-					continue
-				}
-
-				certDigest := localtls.CertFingerprint(cert)
-				if joinToken.Fingerprint != certDigest {
-					return fmt.Errorf(i18n.G("Certificate fingerprint mismatch between join token and cluster member %q"), clusterAddress)
-				}
-
-				config.Cluster.ClusterCertificate = string(pem.EncodeToMemory(&pem.Block{Type: "CERTIFICATE", Bytes: cert.Raw}))
-
-				break // We've found a working cluster member.
-			}
-
-			if config.Cluster.ClusterCertificate == "" {
-				return fmt.Errorf(i18n.G("Unable to connect to any of the cluster members specified in join token"))
-			}
-
-			// Pass the raw join token.
-			config.Cluster.ClusterToken = clusterJoinToken
-
-			// Confirm wiping
-			clusterWipeMember, err := c.global.asker.AskBool(i18n.G("All existing data is lost when joining a cluster, continue?")+" (yes/no) [default=no] ", "no")
-			if err != nil {
-				return err
-			}
-
-			if !clusterWipeMember {
-				return fmt.Errorf(i18n.G("User aborted configuration"))
-			}
-
-			// Connect to existing cluster
-			serverCert, err := internalUtil.LoadServerCert(internalUtil.VarPath(""))
-			if err != nil {
-				return err
-			}
-
-			err = c.setupClusterTrust(serverCert, config.Cluster.ServerName, config.Cluster.ClusterAddress, config.Cluster.ClusterCertificate, config.Cluster.ClusterToken)
-			if err != nil {
-				return fmt.Errorf(i18n.G("Failed to setup trust relationship with cluster: %w"), err)
-			}
-
-			// Now we have setup trust, don't send to server, otherwise it will try and setup trust
-			// again and if using a one-time join token, will fail.
-			config.Cluster.ClusterToken = ""
-
-			// Client parameters to connect to the target cluster member.
-			args := &incus.ConnectionArgs{
-				TLSClientCert: string(serverCert.PublicKey()),
-				TLSClientKey:  string(serverCert.PrivateKey()),
-				TLSServerCert: string(config.Cluster.ClusterCertificate),
-				UserAgent:     version.UserAgent,
-			}
-
-			client, err := incus.ConnectIncus(fmt.Sprintf("https://%s", config.Cluster.ClusterAddress), args)
-			if err != nil {
-				return err
-			}
-
-			// Get the list of required member config keys.
-			cluster, _, err := client.GetCluster()
-			if err != nil {
-				return fmt.Errorf(i18n.G("Failed to retrieve cluster information: %w"), err)
-			}
-
-			for i, config := range cluster.MemberConfig {
-				question := fmt.Sprintf(i18n.G("Choose %s:")+" ", config.Description)
-
-				// Allow for empty values.
-				configValue, err := c.global.asker.AskString(question, "", validate.Optional())
-				if err != nil {
-					return err
-				}
-
-				cluster.MemberConfig[i].Value = configValue
-			}
-
-			config.Cluster.MemberConfig = cluster.MemberConfig
-		} else {
-			// Ask for server name since no token is provided
-			err = askForServerName()
-			if err != nil {
-				return err
-			}
-		}
-	}
-
-	return nil
+	return config, nil
 }
 
 func (c *cmdAdminInit) askNetworking(config *api.InitPreseed, d incus.InstanceServer) error {
@@ -341,19 +145,19 @@ func (c *cmdAdminInit) askNetworking(config *api.InitPreseed, d incus.InstanceSe
 
 	for {
 		// Define the network
-		net := api.InitNetworksProjectPost{}
-		net.Config = map[string]string{}
-		net.Project = api.ProjectDefaultName
+		network := api.InitNetworksProjectPost{}
+		network.Config = map[string]string{}
+		network.Project = api.ProjectDefaultName
 
 		// Network name
-		net.Name, err = c.global.asker.AskString(i18n.G("What should the new bridge be called?")+" [default=incusbr0]: ", "incusbr0", validate.IsNetworkName)
+		network.Name, err = c.global.asker.AskString(i18n.G("What should the new bridge be called?")+" [default=incusbr0]: ", "incusbr0", validate.IsInterfaceName)
 		if err != nil {
 			return err
 		}
 
-		_, _, err = d.GetNetwork(net.Name)
+		_, _, err = d.GetNetwork(network.Name)
 		if err == nil {
-			fmt.Printf(i18n.G("The requested network bridge \"%s\" already exists. Please choose another name.")+"\n", net.Name)
+			fmt.Printf(i18n.G("The requested network bridge \"%s\" already exists. Please choose another name.")+"\n", network.Name)
 			continue
 		}
 
@@ -361,11 +165,11 @@ func (c *cmdAdminInit) askNetworking(config *api.InitPreseed, d incus.InstanceSe
 		config.Server.Profiles[0].Devices["eth0"] = map[string]string{
 			"type":    "nic",
 			"name":    "eth0",
-			"network": net.Name,
+			"network": network.Name,
 		}
 
 		// IPv4
-		net.Config["ipv4.address"], err = c.global.asker.AskString(i18n.G("What IPv4 address should be used?")+" (CIDR subnet notation, “auto” or “none”) [default=auto]: ", "auto", func(value string) error {
+		network.Config["ipv4.address"], err = c.global.asker.AskString(i18n.G("What IPv4 address should be used?")+" (CIDR subnet notation, “auto” or “none”) [default=auto]: ", "auto", func(value string) error {
 			if slices.Contains([]string{"auto", "none"}, value) {
 				return nil
 			}
@@ -376,17 +180,17 @@ func (c *cmdAdminInit) askNetworking(config *api.InitPreseed, d incus.InstanceSe
 			return err
 		}
 
-		if !slices.Contains([]string{"auto", "none"}, net.Config["ipv4.address"]) {
+		if !slices.Contains([]string{"auto", "none"}, network.Config["ipv4.address"]) {
 			netIPv4UseNAT, err := c.global.asker.AskBool(i18n.G("Would you like to NAT IPv4 traffic on your bridge?")+" [default=yes]: ", "yes")
 			if err != nil {
 				return err
 			}
 
-			net.Config["ipv4.nat"] = fmt.Sprintf("%v", netIPv4UseNAT)
+			network.Config["ipv4.nat"] = fmt.Sprintf("%v", netIPv4UseNAT)
 		}
 
 		// IPv6
-		net.Config["ipv6.address"], err = c.global.asker.AskString(i18n.G("What IPv6 address should be used?")+" (CIDR subnet notation, “auto” or “none”) [default=auto]: ", "auto", func(value string) error {
+		network.Config["ipv6.address"], err = c.global.asker.AskString(i18n.G("What IPv6 address should be used?")+" (CIDR subnet notation, “auto” or “none”) [default=auto]: ", "auto", func(value string) error {
 			if slices.Contains([]string{"auto", "none"}, value) {
 				return nil
 			}
@@ -397,17 +201,17 @@ func (c *cmdAdminInit) askNetworking(config *api.InitPreseed, d incus.InstanceSe
 			return err
 		}
 
-		if !slices.Contains([]string{"auto", "none"}, net.Config["ipv6.address"]) {
+		if !slices.Contains([]string{"auto", "none"}, network.Config["ipv6.address"]) {
 			netIPv6UseNAT, err := c.global.asker.AskBool(i18n.G("Would you like to NAT IPv6 traffic on your bridge?")+" [default=yes]: ", "yes")
 			if err != nil {
 				return err
 			}
 
-			net.Config["ipv6.nat"] = fmt.Sprintf("%v", netIPv6UseNAT)
+			network.Config["ipv6.nat"] = fmt.Sprintf("%v", netIPv6UseNAT)
 		}
 
 		// Add the new network
-		config.Server.Networks = append(config.Server.Networks, net)
+		config.Server.Networks = append(config.Server.Networks, network)
 		break
 	}
 
@@ -455,43 +259,13 @@ func (c *cmdAdminInit) askStorage(config *api.InitPreseed, d incus.InstanceServe
 	return c.askStoragePool(config, d, server, internalUtil.PoolTypeAny)
 }
 
-func (c *cmdAdminInit) setupClusterTrust(serverCert *localtls.CertInfo, serverName string, targetAddress string, targetCert string, targetToken string) error {
-	// Connect to the target cluster node.
-	args := &incus.ConnectionArgs{
-		TLSServerCert: targetCert,
-		UserAgent:     version.UserAgent,
-	}
-
-	target, err := incus.ConnectIncus(fmt.Sprintf("https://%s", targetAddress), args)
-	if err != nil {
-		return fmt.Errorf(i18n.G("Failed to connect to target cluster node %q: %w"), targetAddress, err)
-	}
-
-	cert, err := localtls.GenerateTrustCertificate(serverCert, serverName)
-	if err != nil {
-		return fmt.Errorf(i18n.G("Failed generating trust certificate: %w"), err)
-	}
-
-	post := api.CertificatesPost{
-		CertificatePut: cert.CertificatePut,
-		TrustToken:     targetToken,
-	}
-
-	err = target.CreateCertificate(post)
-	if err != nil && !api.StatusErrorCheck(err, http.StatusConflict) {
-		return fmt.Errorf(i18n.G("Failed to add server cert to cluster: %w"), err)
-	}
-
-	return nil
-}
-
 func (c *cmdAdminInit) askStoragePool(config *api.InitPreseed, d incus.InstanceServer, server *api.Server, poolType internalUtil.PoolType) error {
 	// Figure out the preferred storage driver
 	availableBackends := linux.AvailableStorageDrivers(internalUtil.VarPath(), server.Environment.StorageSupportedDrivers, poolType)
 
 	if len(availableBackends) == 0 {
 		if poolType != internalUtil.PoolTypeAny {
-			return fmt.Errorf(i18n.G("No storage backends available"))
+			return errors.New(i18n.G("No storage backends available"))
 		}
 
 		return fmt.Errorf(i18n.G("No %s storage backends available"), poolType)
@@ -615,7 +389,8 @@ func (c *cmdAdminInit) askStoragePool(config *api.InitPreseed, d incus.InstanceS
 		}
 
 		if poolCreate {
-			if pool.Driver == "ceph" {
+			switch pool.Driver {
+			case "ceph":
 				// Ask for the name of the cluster
 				pool.Config["ceph.cluster_name"], err = c.global.asker.AskString(i18n.G("Name of the existing CEPH cluster")+" [default=ceph]: ", "ceph", nil)
 				if err != nil {
@@ -633,7 +408,8 @@ func (c *cmdAdminInit) askStoragePool(config *api.InitPreseed, d incus.InstanceS
 				if err != nil {
 					return err
 				}
-			} else if pool.Driver == "cephfs" {
+
+			case "cephfs":
 				// Ask for the name of the cluster
 				pool.Config["cephfs.cluster_name"], err = c.global.asker.AskString(i18n.G("Name of the existing CEPHfs cluster")+" [default=ceph]: ", "ceph", nil)
 				if err != nil {
@@ -645,13 +421,15 @@ func (c *cmdAdminInit) askStoragePool(config *api.InitPreseed, d incus.InstanceS
 				if err != nil {
 					return err
 				}
-			} else if pool.Driver == "lvmcluster" {
+
+			case "lvmcluster":
 				// Ask for the volume group
 				pool.Config["source"], err = c.global.asker.AskString(i18n.G("Name of the shared LVM volume group:")+" ", "", nil)
 				if err != nil {
 					return err
 				}
-			} else {
+
+			default:
 				useEmptyBlockDev, err := c.global.asker.AskBool(i18n.G("Would you like to use an existing empty block device (e.g. a disk or partition)?")+" (yes/no) [default=no]: ", "no")
 				if err != nil {
 					return err
@@ -676,14 +454,7 @@ func (c *cmdAdminInit) askStoragePool(config *api.InitPreseed, d incus.InstanceS
 					}
 
 					/* choose 5 GiB < x < 30GiB, where x is 20% of the disk size */
-					defaultSize := uint64(st.Frsize) * st.Blocks / (1024 * 1024 * 1024) / 5
-					if defaultSize > 30 {
-						defaultSize = 30
-					}
-
-					if defaultSize < 5 {
-						defaultSize = 5
-					}
+					defaultSize := max(min(uint64(st.Frsize)*st.Blocks/(1024*1024*1024)/5, 30), 5)
 
 					pool.Config["size"], err = c.global.asker.AskString(
 						fmt.Sprintf(i18n.G("Size in GiB of the new loop device")+" (1GiB minimum) [default=%dGiB]: ", defaultSize),
@@ -697,7 +468,7 @@ func (c *cmdAdminInit) askStoragePool(config *api.InitPreseed, d incus.InstanceS
 							}
 
 							if result < 1 {
-								return fmt.Errorf(i18n.G("Minimum size is 1GiB"))
+								return errors.New(i18n.G("Minimum size is 1GiB"))
 							}
 
 							return nil
@@ -751,7 +522,7 @@ and make sure that your user can see and run the "thin_check" command before run
 				}
 
 				if !lvmContinueNoThin {
-					return fmt.Errorf(i18n.G("The LVM thin provisioning tools couldn't be found on the system"))
+					return errors.New(i18n.G("The LVM thin provisioning tools couldn't be found on the system"))
 				}
 
 				pool.Config["lvm.use_thinpool"] = "false"
@@ -765,7 +536,7 @@ and make sure that your user can see and run the "thin_check" command before run
 	return nil
 }
 
-func (c *cmdAdminInit) askDaemon(config *api.InitPreseed, d incus.InstanceServer, server *api.Server) error {
+func (c *cmdAdminInit) askDaemon(config *api.InitPreseed, server *api.Server) error {
 	// Detect lack of uid/gid
 	if linux.RunningInUserNS() {
 		fmt.Print("\n" + i18n.G(`We detected that you are running inside an unprivileged container.
