@@ -14,13 +14,13 @@ import (
 	"github.com/spf13/cobra"
 
 	incus "github.com/lxc/incus/v6/client"
-	cli "github.com/lxc/incus/v6/internal/cmd"
 	"github.com/lxc/incus/v6/internal/i18n"
 	internalUtil "github.com/lxc/incus/v6/internal/util"
 	"github.com/lxc/incus/v6/internal/version"
 	"github.com/lxc/incus/v6/shared/api"
 	"github.com/lxc/incus/v6/shared/ask"
 	config "github.com/lxc/incus/v6/shared/cliconfig"
+	cli "github.com/lxc/incus/v6/shared/cmd"
 	"github.com/lxc/incus/v6/shared/logger"
 	"github.com/lxc/incus/v6/shared/util"
 )
@@ -563,17 +563,13 @@ func (c *cmdGlobal) parseServers(remotes ...string) ([]remoteResource, error) {
 }
 
 func (c *cmdGlobal) checkArgs(cmd *cobra.Command, args []string, minArgs int, maxArgs int) (bool, error) {
-	if len(args) < minArgs || (maxArgs != -1 && len(args) > maxArgs) {
-		_ = cmd.Help()
-
-		if len(args) == 0 {
-			return true, nil
-		}
-
-		return true, errors.New(i18n.G("Invalid number of arguments"))
+	exit, err := cli.CheckArgs(cmd, args, minArgs, maxArgs)
+	if err == cli.ErrBadArgs {
+		// Use translated error message.
+		return exit, errors.New(i18n.G("Invalid number of arguments"))
 	}
 
-	return false, nil
+	return exit, err
 }
 
 // Return the default list format if the user configured it, otherwise just return "table".
