@@ -15,9 +15,9 @@ import (
 	"github.com/spf13/cobra"
 	"gopkg.in/yaml.v2"
 
-	cli "github.com/lxc/incus/v6/internal/cmd"
 	"github.com/lxc/incus/v6/internal/i18n"
 	"github.com/lxc/incus/v6/shared/api"
+	cli "github.com/lxc/incus/v6/shared/cmd"
 	"github.com/lxc/incus/v6/shared/termios"
 )
 
@@ -33,7 +33,7 @@ type cmdProfile struct {
 // Command returns a cobra.Command for use with (*cobra.Command).AddCommand.
 func (c *cmdProfile) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = usage("profile")
+	cmd.Use = cli.Usage("profile")
 	cmd.Short = i18n.G("Manage profiles")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Manage profiles`))
@@ -109,7 +109,7 @@ type cmdProfileAdd struct {
 // Command returns a cobra.Command for use with (*cobra.Command).AddCommand.
 func (c *cmdProfileAdd) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = usage("add", i18n.G("[<remote>:]<instance> <profile>"))
+	cmd.Use = cli.Usage("add", i18n.G("[<remote>:]<instance> <profile>"))
 	cmd.Short = i18n.G("Add profiles to instances")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Add profiles to instances`))
@@ -185,7 +185,7 @@ type cmdProfileAssign struct {
 // Command returns a cobra.Command for use with (*cobra.Command).AddCommand.
 func (c *cmdProfileAssign) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = usage("assign", i18n.G("[<remote>:]<instance> <profiles>"))
+	cmd.Use = cli.Usage("assign", i18n.G("[<remote>:]<instance> <profiles>"))
 	cmd.Aliases = []string{"apply"}
 	cmd.Short = i18n.G("Assign sets of profiles to instances")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
@@ -278,7 +278,7 @@ type cmdProfileCopy struct {
 // Command returns a cobra.Command for use with (*cobra.Command).AddCommand.
 func (c *cmdProfileCopy) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = usage("copy", i18n.G("[<remote>:]<profile> [<remote>:]<profile>"))
+	cmd.Use = cli.Usage("copy", i18n.G("[<remote>:]<profile> [<remote>:]<profile>"))
 	cmd.Aliases = []string{"cp"}
 	cmd.Short = i18n.G("Copy profiles")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
@@ -365,7 +365,7 @@ type cmdProfileCreate struct {
 // Command returns a cobra.Command for use with (*cobra.Command).AddCommand.
 func (c *cmdProfileCreate) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = usage("create", i18n.G("[<remote>:]<profile>"))
+	cmd.Use = cli.Usage("create", i18n.G("[<remote>:]<profile>"))
 	cmd.Short = i18n.G("Create profiles")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Create profiles`))
@@ -455,7 +455,7 @@ type cmdProfileDelete struct {
 // Command returns a cobra.Command for use with (*cobra.Command).AddCommand.
 func (c *cmdProfileDelete) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = usage("delete", i18n.G("[<remote>:]<profile>"))
+	cmd.Use = cli.Usage("delete", i18n.G("[<remote>:]<profile> [[<remote>:]<profile>...]"))
 	cmd.Aliases = []string{"rm"}
 	cmd.Short = i18n.G("Delete profiles")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
@@ -464,11 +464,7 @@ func (c *cmdProfileDelete) Command() *cobra.Command {
 	cmd.RunE = c.Run
 
 	cmd.ValidArgsFunction = func(_ *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
-		if len(args) == 0 {
-			return c.global.cmpProfiles(toComplete, true)
-		}
-
-		return nil, cobra.ShellCompDirectiveNoFileComp
+		return c.global.cmpProfiles(toComplete, true)
 	}
 
 	return cmd
@@ -477,31 +473,31 @@ func (c *cmdProfileDelete) Command() *cobra.Command {
 // Run runs the actual command logic.
 func (c *cmdProfileDelete) Run(cmd *cobra.Command, args []string) error {
 	// Quick checks.
-	exit, err := c.global.checkArgs(cmd, args, 1, 1)
+	exit, err := c.global.checkArgs(cmd, args, 1, -1)
 	if exit {
 		return err
 	}
 
 	// Parse remote
-	resources, err := c.global.parseServers(args[0])
+	resources, err := c.global.parseServers(args...)
 	if err != nil {
 		return err
 	}
 
-	resource := resources[0]
+	for _, resource := range resources {
+		if resource.name == "" {
+			return errors.New(i18n.G("Missing profile name"))
+		}
 
-	if resource.name == "" {
-		return errors.New(i18n.G("Missing profile name"))
-	}
+		// Delete the profile
+		err = resource.server.DeleteProfile(resource.name)
+		if err != nil {
+			return err
+		}
 
-	// Delete the profile
-	err = resource.server.DeleteProfile(resource.name)
-	if err != nil {
-		return err
-	}
-
-	if !c.global.flagQuiet {
-		fmt.Printf(i18n.G("Profile %s deleted")+"\n", resource.name)
+		if !c.global.flagQuiet {
+			fmt.Printf(i18n.G("Profile %s deleted")+"\n", resource.name)
+		}
 	}
 
 	return nil
@@ -516,7 +512,7 @@ type cmdProfileEdit struct {
 // Command returns a cobra.Command for use with (*cobra.Command).AddCommand.
 func (c *cmdProfileEdit) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = usage("edit", i18n.G("[<remote>:]<profile>"))
+	cmd.Use = cli.Usage("edit", i18n.G("[<remote>:]<profile>"))
 	cmd.Short = i18n.G("Edit profile configurations as YAML")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Edit profile configurations as YAML`))
@@ -606,7 +602,7 @@ func (c *cmdProfileEdit) Run(cmd *cobra.Command, args []string) error {
 	}
 
 	// Spawn the editor
-	content, err := textEditor("", []byte(c.helpTemplate()+"\n\n"+string(data)))
+	content, err := cli.TextEditor("", []byte(c.helpTemplate()+"\n\n"+string(data)))
 	if err != nil {
 		return err
 	}
@@ -629,7 +625,7 @@ func (c *cmdProfileEdit) Run(cmd *cobra.Command, args []string) error {
 				return err
 			}
 
-			content, err = textEditor("", content)
+			content, err = cli.TextEditor("", content)
 			if err != nil {
 				return err
 			}
@@ -654,7 +650,7 @@ type cmdProfileGet struct {
 // Command returns a cobra.Command for use with (*cobra.Command).AddCommand.
 func (c *cmdProfileGet) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = usage("get", i18n.G("[<remote>:]<profile> <key>"))
+	cmd.Use = cli.Usage("get", i18n.G("[<remote>:]<profile> <key>"))
 	cmd.Short = i18n.G("Get values for profile configuration keys")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Get values for profile configuration keys`))
@@ -731,7 +727,7 @@ type cmdProfileList struct {
 // Command returns a cobra.Command for use with (*cobra.Command).AddCommand.
 func (c *cmdProfileList) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = usage("list", i18n.G("[<remote>:] [<filter>...]"))
+	cmd.Use = cli.Usage("list", i18n.G("[<remote>:] [<filter>...]"))
 	cmd.Aliases = []string{"ls"}
 	cmd.Short = i18n.G("List profiles")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
@@ -923,7 +919,7 @@ type cmdProfileRemove struct {
 // Command returns a cobra.Command for use with (*cobra.Command).AddCommand.
 func (c *cmdProfileRemove) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = usage("remove", i18n.G("[<remote>:]<instance> <profile>"))
+	cmd.Use = cli.Usage("remove", i18n.G("[<remote>:]<instance> <profile>"))
 	cmd.Short = i18n.G("Remove profiles from instances")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Remove profiles from instances`))
@@ -1012,7 +1008,7 @@ type cmdProfileRename struct {
 // Command returns a cobra.Command for use with (*cobra.Command).AddCommand.
 func (c *cmdProfileRename) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = usage("rename", i18n.G("[<remote>:]<profile> <new-name>"))
+	cmd.Use = cli.Usage("rename", i18n.G("[<remote>:]<profile> <new-name>"))
 	cmd.Aliases = []string{"mv"}
 	cmd.Short = i18n.G("Rename profiles")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
@@ -1075,7 +1071,7 @@ type cmdProfileSet struct {
 // Command returns a cobra.Command for use with (*cobra.Command).AddCommand.
 func (c *cmdProfileSet) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = usage("set", i18n.G("[<remote>:]<profile> <key>=<value>..."))
+	cmd.Use = cli.Usage("set", i18n.G("[<remote>:]<profile> <key>=<value>..."))
 	cmd.Short = i18n.G("Set profile configuration keys")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Set profile configuration keys
@@ -1164,7 +1160,7 @@ type cmdProfileShow struct {
 // Command returns a cobra.Command for use with (*cobra.Command).AddCommand.
 func (c *cmdProfileShow) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = usage("show", i18n.G("[<remote>:]<profile>"))
+	cmd.Use = cli.Usage("show", i18n.G("[<remote>:]<profile>"))
 	cmd.Short = i18n.G("Show profile configurations")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Show profile configurations`))
@@ -1230,7 +1226,7 @@ type cmdProfileUnset struct {
 // Command returns a cobra.Command for use with (*cobra.Command).AddCommand.
 func (c *cmdProfileUnset) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = usage("unset", i18n.G("[<remote>:]<profile> <key>"))
+	cmd.Use = cli.Usage("unset", i18n.G("[<remote>:]<profile> <key>"))
 	cmd.Short = i18n.G("Unset profile configuration keys")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Unset profile configuration keys`))
