@@ -2,7 +2,6 @@ package main
 
 import (
 	"context"
-	"errors"
 	"fmt"
 	"io"
 	"math/rand"
@@ -22,7 +21,6 @@ import (
 	"github.com/lxc/incus/v6/internal/instance"
 	"github.com/lxc/incus/v6/shared/api"
 	config "github.com/lxc/incus/v6/shared/cliconfig"
-	"github.com/lxc/incus/v6/shared/revert"
 	"github.com/lxc/incus/v6/shared/termios"
 	localtls "github.com/lxc/incus/v6/shared/tls"
 )
@@ -337,14 +335,6 @@ func readEnvironmentFile(path string) (map[string]string, error) {
 	return envMap, nil
 }
 
-func usage(name string, args ...string) string {
-	if len(args) == 0 {
-		return name
-	}
-
-	return name + " " + args[0]
-}
-
 // instancesExist iterates over a list of instances (or snapshots) and checks that they exist.
 func instancesExist(resources []remoteResource) error {
 	for _, resource := range resources {
@@ -497,90 +487,6 @@ func getImgInfo(d incus.InstanceServer, conf *config.Config, imgRemote string, i
 	return imgRemoteServer, imgInfo, nil
 }
 
-// Spawn the editor with a temporary YAML file for editing configs.
-func textEditor(inPath string, inContent []byte) ([]byte, error) {
-	var f *os.File
-	var err error
-	var path string
-
-	// Detect the text editor to use
-	editor := os.Getenv("VISUAL")
-	if editor == "" {
-		editor = os.Getenv("EDITOR")
-		if editor == "" {
-			for _, p := range []string{"editor", "vi", "emacs", "nano"} {
-				_, err := exec.LookPath(p)
-				if err == nil {
-					editor = p
-					break
-				}
-			}
-			if editor == "" {
-				return []byte{}, errors.New(i18n.G("No text editor found, please set the EDITOR environment variable"))
-			}
-		}
-	}
-
-	if inPath == "" {
-		// If provided input, create a new file
-		f, err = os.CreateTemp("", "incus_editor_")
-		if err != nil {
-			return []byte{}, err
-		}
-
-		reverter := revert.New()
-		defer reverter.Fail()
-
-		reverter.Add(func() {
-			_ = f.Close()
-			_ = os.Remove(f.Name())
-		})
-
-		err = os.Chmod(f.Name(), 0o600)
-		if err != nil {
-			return []byte{}, err
-		}
-
-		_, err = f.Write(inContent)
-		if err != nil {
-			return []byte{}, err
-		}
-
-		err = f.Close()
-		if err != nil {
-			return []byte{}, err
-		}
-
-		path = fmt.Sprintf("%s.yaml", f.Name())
-		err = os.Rename(f.Name(), path)
-		if err != nil {
-			return []byte{}, err
-		}
-
-		reverter.Success()
-		reverter.Add(func() { _ = os.Remove(path) })
-	} else {
-		path = inPath
-	}
-
-	cmdParts := strings.Fields(editor)
-	cmd := exec.Command(cmdParts[0], append(cmdParts[1:], path)...)
-	cmd.Stdin = os.Stdin
-	cmd.Stdout = os.Stdout
-	cmd.Stderr = os.Stderr
-	err = cmd.Run()
-	if err != nil {
-		return []byte{}, err
-	}
-
-	content, err := os.ReadFile(path)
-	if err != nil {
-		return []byte{}, err
-	}
-
-	return content, nil
-}
-
 // removeElementsFromSlice returns a slice equivalent to removing the given elements from the given list.
 // Elements not present in the list are ignored.
 func removeElementsFromSlice[T comparable](list []T, elements ...T) []T {
