@@ -3,6 +3,7 @@ package main
 import (
 	"bytes"
 	"context"
+	"errors"
 	"fmt"
 	"io"
 	"os"
@@ -22,6 +23,7 @@ import (
 	"github.com/lxc/incus/v6/internal/server/project"
 	"github.com/lxc/incus/v6/internal/server/state"
 	storagePools "github.com/lxc/incus/v6/internal/server/storage"
+	"github.com/lxc/incus/v6/internal/server/storage/drivers"
 	"github.com/lxc/incus/v6/internal/server/task"
 	internalUtil "github.com/lxc/incus/v6/internal/util"
 	"github.com/lxc/incus/v6/shared/api"
@@ -39,8 +41,8 @@ func backupCreate(s *state.State, args db.InstanceBackup, sourceInst instance.In
 	l.Debug("Instance backup started")
 	defer l.Debug("Instance backup finished")
 
-	revert := revert.New()
-	defer revert.Fail()
+	reverter := revert.New()
+	defer reverter.Fail()
 
 	// Get storage pool.
 	pool, err := storagePools.LoadByInstance(s, sourceInst)
@@ -58,14 +60,14 @@ func backupCreate(s *state.State, args db.InstanceBackup, sourceInst instance.In
 		return tx.CreateInstanceBackup(ctx, args)
 	})
 	if err != nil {
-		if err == db.ErrAlreadyDefined {
+		if errors.Is(err, db.ErrAlreadyDefined) {
 			return fmt.Errorf("Backup %q already exists", args.Name)
 		}
 
 		return fmt.Errorf("Insert backup info into database: %w", err)
 	}
 
-	revert.Add(func() {
+	reverter.Add(func() {
 		_ = s.DB.Cluster.Transaction(context.Background(), func(ctx context.Context, tx *db.ClusterTx) error {
 			return tx.DeleteInstanceBackup(ctx, args.Name)
 		})
@@ -113,7 +115,7 @@ func backupCreate(s *state.State, args db.InstanceBackup, sourceInst instance.In
 			return err
 		}
 
-		revert.Add(func() { _ = os.Remove(backupsPath) })
+		reverter.Add(func() { _ = os.Remove(backupsPath) })
 	}
 
 	target := internalUtil.VarPath("backups", "instances", project.Instance(sourceInst.Project().Name, b.Name()))
@@ -126,7 +128,7 @@ func backupCreate(s *state.State, args db.InstanceBackup, sourceInst instance.In
 	}
 
 	defer func() { _ = tarFileWriter.Close() }()
-	revert.Add(func() { _ = os.Remove(target) })
+	reverter.Add(func() { _ = os.Remove(target) })
 
 	// Get IDMap to unshift container as the tarball is created.
 	var idmapSet *idmap.Set
@@ -150,14 +152,7 @@ func backupCreate(s *state.State, args db.InstanceBackup, sourceInst instance.In
 	backupProgressWriter := &ioprogress.ProgressWriter{
 		Tracker: &ioprogress.ProgressTracker{
 			Handler: func(value, speed int64) {
-				meta := op.Metadata()
-				if meta == nil {
-					meta = make(map[string]any)
-				}
-
-				progressText := fmt.Sprintf("%s (%s/s)", units.GetByteSizeString(value, 2), units.GetByteSizeString(speed, 2))
-				meta["create_backup_progress"] = progressText
-				_ = op.UpdateMetadata(meta)
+				_ = op.ExtendMetadata(map[string]any{"create_backup_progress": fmt.Sprintf("%s (%s/s)", units.GetByteSizeString(value, 2), units.GetByteSizeString(speed, 2))})
 			},
 		},
 	}
@@ -222,7 +217,7 @@ func backupCreate(s *state.State, args db.InstanceBackup, sourceInst instance.In
 		return fmt.Errorf("Error closing tar file: %w", err)
 	}
 
-	revert.Success()
+	reverter.Success()
 	s.Events.SendLifecycle(sourceInst.Project().Name, lifecycle.InstanceBackupCreated.Event(args.Name, b.Instance(), nil))
 
 	return nil
@@ -238,12 +233,12 @@ func backupWriteIndex(sourceInst instance.Instance, pool storagePools.Pool, opti
 
 	backupType := backup.InstanceTypeToBackupType(api.InstanceType(sourceInst.Type().String()))
 	if backupType == backup.TypeUnknown {
-		return fmt.Errorf("Unrecognised instance type for backup type conversion")
+		return errors.New("Unrecognised instance type for backup type conversion")
 	}
 
 	// We only write backup files out for actual instances.
 	if sourceInst.IsSnapshot() {
-		return fmt.Errorf("Cannot generate backup config for snapshots")
+		return errors.New("Cannot generate backup config for snapshots")
 	}
 
 	// Immediately return if the instance directory doesn't exist yet.
@@ -393,8 +388,8 @@ func volumeBackupCreate(s *state.State, args db.StoragePoolVolumeBackup, project
 	l.Debug("Volume backup started")
 	defer l.Debug("Volume backup finished")
 
-	revert := revert.New()
-	defer revert.Fail()
+	reverter := revert.New()
+	defer reverter.Fail()
 
 	// Get storage pool.
 	pool, err := storagePools.LoadByName(s, poolName)
@@ -402,8 +397,24 @@ func volumeBackupCreate(s *state.State, args db.StoragePoolVolumeBackup, project
 		return fmt.Errorf("Failed loading storage pool %q: %w", poolName, err)
 	}
 
-	// Ignore requests for optimized backups when pool driver doesn't support it.
-	if args.OptimizedStorage && !pool.Driver().Info().OptimizedBackups {
+	// Get the DB volume.
+	volume, err := storagePools.VolumeDBGet(pool, projectName, volumeName, drivers.VolumeTypeCustom)
+	if err != nil {
+		return fmt.Errorf("Failed getting volume record: %w", err)
+	}
+
+	contentDBType, err := storagePools.VolumeContentTypeNameToContentType(volume.ContentType)
+	if err != nil {
+		return err
+	}
+
+	contentType, err := storagePools.VolumeDBContentTypeToContentType(contentDBType)
+	if err != nil {
+		return err
+	}
+
+	// Ignore requests for optimized backups when pool driver doesn't support it, or when backing up ISO volumes.
+	if args.OptimizedStorage && (!pool.Driver().Info().OptimizedBackups || contentType == drivers.ContentTypeISO) {
 		args.OptimizedStorage = false
 	}
 
@@ -412,14 +423,14 @@ func volumeBackupCreate(s *state.State, args db.StoragePoolVolumeBackup, project
 		return tx.CreateStoragePoolVolumeBackup(ctx, args)
 	})
 	if err != nil {
-		if err == db.ErrAlreadyDefined {
+		if errors.Is(err, db.ErrAlreadyDefined) {
 			return fmt.Errorf("Backup %q already exists", args.Name)
 		}
 
 		return fmt.Errorf("Failed creating backup record: %w", err)
 	}
 
-	revert.Add(func() {
+	reverter.Add(func() {
 		_ = s.DB.Cluster.Transaction(context.Background(), func(ctx context.Context, tx *db.ClusterTx) error {
 			return tx.DeleteStoragePoolVolumeBackup(ctx, args.Name)
 		})
@@ -454,94 +465,103 @@ func volumeBackupCreate(s *state.State, args db.StoragePoolVolumeBackup, project
 			return err
 		}
 
-		revert.Add(func() { _ = os.Remove(backupsPath) })
+		reverter.Add(func() { _ = os.Remove(backupsPath) })
 	}
 
 	target := internalUtil.VarPath("backups", "custom", pool.Name(), project.StorageVolume(projectName, backupRow.Name))
 
-	// Setup the tarball writer.
-	l.Debug("Opening backup tarball for writing", logger.Ctx{"path": target})
-	tarFileWriter, err := os.OpenFile(target, os.O_CREATE|os.O_WRONLY, 0o600)
+	// Setup the writer.
+	l.Debug("Opening backup file for writing", logger.Ctx{"path": target})
+	fileWriter, err := os.OpenFile(target, os.O_CREATE|os.O_WRONLY, 0o600)
 	if err != nil {
-		return fmt.Errorf("Error opening backup tarball for writing %q: %w", target, err)
+		return fmt.Errorf("Error opening backup file for writing %q: %w", target, err)
 	}
 
-	defer func() { _ = tarFileWriter.Close() }()
-	revert.Add(func() { _ = os.Remove(target) })
+	defer func() { _ = fileWriter.Close() }()
+	reverter.Add(func() { _ = os.Remove(target) })
 
-	// Create the tarball.
-	tarPipeReader, tarPipeWriter := io.Pipe()
-	defer func() { _ = tarPipeWriter.Close() }() // Ensure that go routine below always ends.
-	tarWriter := instancewriter.NewInstanceTarWriter(tarPipeWriter, nil)
+	// If dealing with an ISO volume, we want to return it unaltered.
+	if contentType == drivers.ContentTypeISO {
+		err = pool.BackupCustomVolume(projectName, volumeName, instancewriter.NewInstanceRawWriter(fileWriter), backupRow.OptimizedStorage, !backupRow.VolumeOnly, nil)
+		if err != nil {
+			return fmt.Errorf("Backup create: %w", err)
+		}
+	} else {
+		// Create the tarball.
+		tarPipeReader, tarPipeWriter := io.Pipe()
+		defer func() { _ = tarPipeWriter.Close() }() // Ensure that go routine below always ends.
 
-	// Setup tar writer go routine, with optional compression.
-	tarWriterRes := make(chan error)
-	var compressErr error
+		tarWriter := instancewriter.NewInstanceTarWriter(tarPipeWriter, nil)
 
-	go func(resCh chan<- error) {
-		l.Debug("Started backup tarball writer")
-		defer l.Debug("Finished backup tarball writer")
-		if compress != "none" {
-			compressErr = compressFile(compress, tarPipeReader, tarFileWriter)
+		// Setup tar writer go routine, with optional compression.
+		tarWriterRes := make(chan error)
+		var compressErr error
 
-			// If a compression error occurred, close the tarPipeWriter to end the export.
-			if compressErr != nil {
-				_ = tarPipeWriter.Close()
+		go func(resCh chan<- error) {
+			l.Debug("Started backup tarball writer")
+			defer l.Debug("Finished backup tarball writer")
+			if compress != "none" {
+				compressErr = compressFile(compress, tarPipeReader, fileWriter)
+
+				// If a compression error occurred, close the tarPipeWriter to end the export.
+				if compressErr != nil {
+					_ = tarPipeWriter.Close()
+				}
+			} else {
+				_, err = io.Copy(fileWriter, tarPipeReader)
 			}
-		} else {
-			_, err = io.Copy(tarFileWriter, tarPipeReader)
+
+			resCh <- err
+		}(tarWriterRes)
+
+		// Write index file.
+		l.Debug("Adding backup index file")
+		err = volumeBackupWriteIndex(projectName, volumeName, pool, backupRow.OptimizedStorage, !backupRow.VolumeOnly, tarWriter)
+
+		// Check compression errors.
+		if compressErr != nil {
+			return compressErr
 		}
 
-		resCh <- err
-	}(tarWriterRes)
+		// Check backupWriteIndex for errors.
+		if err != nil {
+			return fmt.Errorf("Error writing backup index file: %w", err)
+		}
 
-	// Write index file.
-	l.Debug("Adding backup index file")
-	err = volumeBackupWriteIndex(s, projectName, volumeName, pool, backupRow.OptimizedStorage, !backupRow.VolumeOnly, tarWriter)
+		err = pool.BackupCustomVolume(projectName, volumeName, tarWriter, backupRow.OptimizedStorage, !backupRow.VolumeOnly, nil)
+		if err != nil {
+			return fmt.Errorf("Backup create: %w", err)
+		}
 
-	// Check compression errors.
-	if compressErr != nil {
-		return compressErr
+		// Close off the tarball file.
+		err = tarWriter.Close()
+		if err != nil {
+			return fmt.Errorf("Error closing tarball writer: %w", err)
+		}
+
+		// Close off the pipe writer (this will end the go routine above).
+		err = tarPipeWriter.Close()
+		if err != nil {
+			return fmt.Errorf("Error closing tarball pipe writer: %w", err)
+		}
+
+		err = <-tarWriterRes
+		if err != nil {
+			return fmt.Errorf("Error writing tarball: %w", err)
+		}
 	}
 
-	// Check backupWriteIndex for errors.
+	err = fileWriter.Close()
 	if err != nil {
-		return fmt.Errorf("Error writing backup index file: %w", err)
+		return fmt.Errorf("Error closing backup file: %w", err)
 	}
 
-	err = pool.BackupCustomVolume(projectName, volumeName, tarWriter, backupRow.OptimizedStorage, !backupRow.VolumeOnly, nil)
-	if err != nil {
-		return fmt.Errorf("Backup create: %w", err)
-	}
-
-	// Close off the tarball file.
-	err = tarWriter.Close()
-	if err != nil {
-		return fmt.Errorf("Error closing tarball writer: %w", err)
-	}
-
-	// Close off the tarball pipe writer (this will end the go routine above).
-	err = tarPipeWriter.Close()
-	if err != nil {
-		return fmt.Errorf("Error closing tarball pipe writer: %w", err)
-	}
-
-	err = <-tarWriterRes
-	if err != nil {
-		return fmt.Errorf("Error writing tarball: %w", err)
-	}
-
-	err = tarFileWriter.Close()
-	if err != nil {
-		return fmt.Errorf("Error closing tar file: %w", err)
-	}
-
-	revert.Success()
+	reverter.Success()
 	return nil
 }
 
 // volumeBackupWriteIndex generates an index.yaml file and then writes it to the root of the backup tarball.
-func volumeBackupWriteIndex(s *state.State, projectName string, volumeName string, pool storagePools.Pool, optimized bool, snapshots bool, tarWriter *instancewriter.InstanceTarWriter) error {
+func volumeBackupWriteIndex(projectName string, volumeName string, pool storagePools.Pool, optimized bool, snapshots bool, tarWriter *instancewriter.InstanceTarWriter) error {
 	// Indicate whether the driver will include a driver-specific optimized header.
 	poolDriverOptimizedHeader := false
 	if optimized {
@@ -659,7 +679,7 @@ func bucketBackupCreate(s *state.State, args db.StoragePoolBucketBackup, project
 		return tx.CreateStoragePoolBucketBackup(ctx, args)
 	})
 	if err != nil {
-		if err == db.ErrAlreadyDefined {
+		if errors.Is(err, db.ErrAlreadyDefined) {
 			return fmt.Errorf("Backup %q already exists", args.Name)
 		}
 
@@ -743,7 +763,7 @@ func bucketBackupCreate(s *state.State, args db.StoragePoolBucketBackup, project
 
 	// Write index file.
 	l.Debug("Adding backup index file")
-	err = bucketBackupWriteIndex(s, projectName, bucketName, pool, tarWriter)
+	err = bucketBackupWriteIndex(projectName, bucketName, pool, tarWriter)
 
 	// Check compression errors.
 	if compressErr != nil {
@@ -787,7 +807,7 @@ func bucketBackupCreate(s *state.State, args db.StoragePoolBucketBackup, project
 }
 
 // bucketBackupWriteIndex generates an index.yaml file and then writes it to the root of the backup tarball.
-func bucketBackupWriteIndex(s *state.State, projectName string, bucketName string, pool storagePools.Pool, tarWriter *instancewriter.InstanceTarWriter) error {
+func bucketBackupWriteIndex(projectName string, bucketName string, pool storagePools.Pool, tarWriter *instancewriter.InstanceTarWriter) error {
 	config, err := pool.GenerateBucketBackupConfig(projectName, bucketName, nil)
 	if err != nil {
 		return fmt.Errorf("Failed generating storage backup config: %w", err)
