@@ -3,6 +3,7 @@ package main
 import (
 	"context"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"net/http"
 	"net/url"
@@ -24,9 +25,11 @@ import (
 	"github.com/lxc/incus/v6/internal/server/response"
 	"github.com/lxc/incus/v6/internal/server/state"
 	storagePools "github.com/lxc/incus/v6/internal/server/storage"
+	localUtil "github.com/lxc/incus/v6/internal/server/util"
 	internalUtil "github.com/lxc/incus/v6/internal/util"
 	"github.com/lxc/incus/v6/internal/version"
 	"github.com/lxc/incus/v6/shared/api"
+	"github.com/lxc/incus/v6/shared/validate"
 )
 
 var storagePoolBucketBackupsCmd = APIEndpoint{
@@ -160,48 +163,74 @@ func storagePoolBucketBackupsGet(d *Daemon, r *http.Request) response.Response {
 		return resp
 	}
 
-	bucketProjectName, err := project.StorageBucketProject(r.Context(), s.DB.Cluster, request.ProjectParam(r))
+	projectName, err := project.StorageBucketProject(r.Context(), s.DB.Cluster, request.ProjectParam(r))
 	if err != nil {
 		return response.SmartError(err)
 	}
 
-	poolName, err := url.PathUnescape(mux.Vars(r)["poolName"])
-	if err != nil {
-		return response.SmartError(err)
-	}
-
-	pool, err := storagePools.LoadByName(s, poolName)
-	if err != nil {
-		return response.SmartError(fmt.Errorf("Failed loading storage pool: %w", err))
-	}
-
-	if !pool.Driver().Info().Buckets {
-		return response.BadRequest(fmt.Errorf("Storage pool does not support buckets"))
-	}
-
+	// Get the name of the storage bucket.
 	bucketName, err := url.PathUnescape(mux.Vars(r)["bucketName"])
 	if err != nil {
 		return response.SmartError(err)
 	}
 
-	targetMember := request.QueryParam(r, "target")
-	memberSpecific := targetMember != ""
+	// Get the name of the storage pool the bucket is supposed to be attached to.
+	poolName, err := url.PathUnescape(mux.Vars(r)["poolName"])
+	if err != nil {
+		return response.SmartError(err)
+	}
+
+	// Get the storage pool itself.
+	var poolID int64
 
-	var bucket *db.StorageBucket
 	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
-		bucket, err = tx.GetStoragePoolBucket(ctx, pool.ID(), bucketProjectName, memberSpecific, bucketName)
+		var err error
+
+		poolID, _, _, err = tx.GetStoragePool(ctx, poolName)
+
 		return err
 	})
 	if err != nil {
 		return response.SmartError(err)
 	}
 
-	u := pool.GetBucketURL(bucket.Name)
-	if u != nil {
-		bucket.S3URL = u.String()
+	// Handle the request.
+	recursion := localUtil.IsRecursionRequest(r)
+
+	var bucketBackups []db.StoragePoolBucketBackup
+
+	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+		bucketBackups, err = tx.GetStoragePoolBucketBackups(ctx, projectName, bucketName, poolID)
+		return err
+	})
+	if err != nil {
+		return response.SmartError(err)
 	}
 
-	return response.SyncResponseETag(true, bucket, bucket.Etag())
+	backups := make([]*backup.BucketBackup, len(bucketBackups))
+
+	for i, b := range bucketBackups {
+		_, backupName, _ := api.GetParentAndSnapshotName(b.Name)
+		backups[i] = backup.NewBucketBackup(s, projectName, poolName, bucketName, b.ID, backupName, b.CreationDate, b.ExpiryDate)
+	}
+
+	resultString := []string{}
+	resultMap := []*api.StorageBucketBackup{}
+
+	for _, entry := range backups {
+		if !recursion {
+			resultString = append(resultString, api.NewURL().Path(version.APIVersion, "storage-pools", poolName, "buckets", bucketName, "backups", strings.Split(entry.Name(), "/")[1]).String())
+		} else {
+			render := entry.Render()
+			resultMap = append(resultMap, render)
+		}
+	}
+
+	if !recursion {
+		return response.SyncResponse(true, resultString)
+	}
+
+	return response.SyncResponse(true, resultMap)
 }
 
 // swagger:operation POST /1.0/storage-pools/{poolName}/buckets/{bucketName}/backups storage storage_pool_buckets_backups_post
@@ -265,7 +294,7 @@ func storagePoolBucketBackupsPost(d *Daemon, r *http.Request) response.Response
 	}
 
 	if !pool.Driver().Info().Buckets {
-		return response.BadRequest(fmt.Errorf("Storage pool does not support buckets"))
+		return response.BadRequest(errors.New("Storage pool does not support buckets"))
 	}
 
 	bucketName, err := url.PathUnescape(mux.Vars(r)["bucketName"])
@@ -276,6 +305,12 @@ func storagePoolBucketBackupsPost(d *Daemon, r *http.Request) response.Response
 	targetMember := request.QueryParam(r, "target")
 	memberSpecific := targetMember != ""
 
+	// Quick checks.
+	err = validate.IsAPIName(bucketName, false)
+	if err != nil {
+		return response.BadRequest(fmt.Errorf("Invalid storage bucket backup name: %w", err))
+	}
+
 	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
 		err := project.AllowBackupCreation(tx, projectName)
 		return err
@@ -356,7 +391,7 @@ func storagePoolBucketBackupsPost(d *Daemon, r *http.Request) response.Response
 
 	// Validate the name.
 	if strings.Contains(req.Name, "/") {
-		return response.BadRequest(fmt.Errorf("Backup names may not contain slashes"))
+		return response.BadRequest(errors.New("Backup names may not contain slashes"))
 	}
 
 	fullName := bucketName + internalInstance.SnapshotDelimiter + req.Name
@@ -460,7 +495,7 @@ func storagePoolBucketBackupGet(d *Daemon, r *http.Request) response.Response {
 	}
 
 	if !pool.Driver().Info().Buckets {
-		return response.BadRequest(fmt.Errorf("Storage pool does not support buckets"))
+		return response.BadRequest(errors.New("Storage pool does not support buckets"))
 	}
 
 	bucketName, err := url.PathUnescape(mux.Vars(r)["bucketName"])
@@ -544,7 +579,7 @@ func storagePoolBucketBackupPost(d *Daemon, r *http.Request) response.Response {
 	}
 
 	if !pool.Driver().Info().Buckets {
-		return response.BadRequest(fmt.Errorf("Storage pool does not support buckets"))
+		return response.BadRequest(errors.New("Storage pool does not support buckets"))
 	}
 
 	bucketName, err := url.PathUnescape(mux.Vars(r)["bucketName"])
@@ -563,9 +598,10 @@ func storagePoolBucketBackupPost(d *Daemon, r *http.Request) response.Response {
 		return response.BadRequest(err)
 	}
 
-	// Validate the name
-	if strings.Contains(req.Name, "/") {
-		return response.BadRequest(fmt.Errorf("Backup names may not contain slashes"))
+	// Quick checks.
+	err = validate.IsAPIName(req.Name, false)
+	if err != nil {
+		return response.BadRequest(fmt.Errorf("Invalid storage bucket backup name: %w", err))
 	}
 
 	oldName := bucketName + internalInstance.SnapshotDelimiter + backupName
@@ -655,7 +691,7 @@ func storagePoolBucketBackupDelete(d *Daemon, r *http.Request) response.Response
 	}
 
 	if !pool.Driver().Info().Buckets {
-		return response.BadRequest(fmt.Errorf("Storage pool does not support buckets"))
+		return response.BadRequest(errors.New("Storage pool does not support buckets"))
 	}
 
 	bucketName, err := url.PathUnescape(mux.Vars(r)["bucketName"])
@@ -749,7 +785,7 @@ func storagePoolBucketBackupExportGet(d *Daemon, r *http.Request) response.Respo
 	}
 
 	if !pool.Driver().Info().Buckets {
-		return response.BadRequest(fmt.Errorf("Storage pool does not support buckets"))
+		return response.BadRequest(errors.New("Storage pool does not support buckets"))
 	}
 
 	bucketName, err := url.PathUnescape(mux.Vars(r)["bucketName"])
@@ -798,8 +834,8 @@ func storagePoolBucketBackupLoadByName(ctx context.Context, s *state.State, proj
 		return nil, err
 	}
 
-	bucketName := strings.Split(backupName, "/")[0]
-	entry := backup.NewBucketBackup(s, projectName, poolName, bucketName, b.ID, b.Name, b.CreationDate, b.ExpiryDate)
+	bucketName, snapName, _ := api.GetParentAndSnapshotName(b.Name)
+	entry := backup.NewBucketBackup(s, projectName, poolName, bucketName, b.ID, snapName, b.CreationDate, b.ExpiryDate)
 
 	return entry, nil
 }
