@@ -4,6 +4,9 @@ package linux
 #include <linux/btrfs.h>
 #include <linux/hidraw.h>
 #include <linux/vhost.h>
+
+#define ZFS_MAX_DATASET_NAME_LEN 256
+#define BLKZNAME _IOR(0x12, 125, char[ZFS_MAX_DATASET_NAME_LEN])
 */
 import "C"
 
@@ -16,4 +19,7 @@ const (
 
 	// IoctlVhostVsockSetGuestCid matches VHOST_VSOCK_SET_GUEST_CID.
 	IoctlVhostVsockSetGuestCid = C.VHOST_VSOCK_SET_GUEST_CID
+
+	// IoctlBlkZname matches BLKZNAME (ZFS specific).
+	IoctlBlkZname = C.BLKZNAME
 )
