@@ -163,6 +163,16 @@ func (c *Config) NetworkOVNSSL() (string, string, string) {
 	return c.m.GetString("network.ovn.ca_cert"), c.m.GetString("network.ovn.client_cert"), c.m.GetString("network.ovn.client_key")
 }
 
+// LinstorControllerConnection returns the Linstor controller connection string.
+func (c *Config) LinstorControllerConnection() string {
+	return c.m.GetString("storage.linstor.controller_connection")
+}
+
+// LinstorSSL returns all three SSL configuration keys needed for a Linstor controller connection.
+func (c *Config) LinstorSSL() (string, string, string) {
+	return c.m.GetString("storage.linstor.ca_cert"), c.m.GetString("storage.linstor.client_cert"), c.m.GetString("storage.linstor.client_key")
+}
+
 // ShutdownTimeout returns the number of minutes to wait for running operation to complete
 // before the server shuts down.
 func (c *Config) ShutdownTimeout() time.Duration {
@@ -314,6 +324,11 @@ func (c *Config) OpenFGA() (apiURL string, apiToken string, storeID string) {
 	return c.m.GetString("openfga.api.url"), c.m.GetString("openfga.api.token"), c.m.GetString("openfga.store.id")
 }
 
+// NetworkHWAddrPattern returns the MAC address pattern used in the cluster.
+func (c *Config) NetworkHWAddrPattern() string {
+	return c.m.GetString("network.hwaddr_pattern")
+}
+
 // Loggers returns a map where the key is the logger name and the value is its type.
 func (c *Config) Loggers() (map[string]string, error) {
 	result := make(map[string]string)
@@ -894,6 +909,17 @@ var ConfigSchema = config.Schema{
 	//  shortdesc: Events to send to the Loki server
 	"loki.types": {Validator: validate.Optional(validate.IsListOf(validate.IsOneOf("lifecycle", "logging", "network-acl"))), Default: "lifecycle,logging", Deprecated: "Use 'logging.*.types' instead"},
 
+	// gendoc:generate(entity=server, group=network, key=network.hwaddr_pattern)
+	// Specify a MAC address template, e.g. `10:66:6a:xx:xx:xx`, to use within the cluster.
+	// Every `x` in the template will be replaced by a random character in `0`–`f`.
+	// Beware of the birthday paradox! A single `xx` block leads to a 10% collision probability with only 8 addresses; for a double `xx:xx` block, 118 addresses; for a triple `xx:xx:xx` block, 1881; for a quadruple `xx:xx:xx:xx` block, 30084. We provide absolutely no guardrail against that.
+	// ---
+	// type: string
+	// scope: global
+	// defaultdesc: `10:66:6a:xx:xx:xx`
+	// shortdesc: MAC address template
+	"network.hwaddr_pattern": {Default: "10:66:6a:xx:xx:xx", Validator: validate.Optional(validate.IsMACPattern)},
+
 	// gendoc:generate(entity=server, group=openfga, key=openfga.api.token)
 	//
 	// ---
@@ -1004,6 +1030,38 @@ var ConfigSchema = config.Schema{
 	//  defaultdesc: Content of `/etc/ovn/key_host` if present
 	//  shortdesc: OVN SSL client key
 	"network.ovn.client_key": {Default: ""},
+
+	// gendoc:generate(entity=server, group=miscellaneous, key=storage.linstor.controller_connection)
+	//
+	// ---
+	//  type: string
+	//  scope: global
+	//  shortdesc: LINSTOR controller connection string
+	"storage.linstor.controller_connection": {Default: ""},
+
+	// gendoc:generate(entity=server, group=miscellaneous, key=storage.linstor.ca_cert)
+	//
+	// ---
+	//  type: string
+	//  scope: global
+	//  shortdesc: LINSTOR SSL certificate authority
+	"storage.linstor.ca_cert": {Default: ""},
+
+	// gendoc:generate(entity=server, group=miscellaneous, key=storage.linstor.client_cert)
+	//
+	// ---
+	//  type: string
+	//  scope: global
+	//  shortdesc: LINSTOR SSL client certificate
+	"storage.linstor.client_cert": {Default: ""},
+
+	// gendoc:generate(entity=server, group=miscellaneous, key=storage.linstor.client_key)
+	//
+	// ---
+	//  type: string
+	//  scope: global
+	//  shortdesc: LINSTOR SSL client key
+	"storage.linstor.client_key": {Default: ""},
 }
 
 func expiryValidator(value string) error {
