@@ -94,7 +94,7 @@ WHERE storage_volumes.id = ?
 
 	err := c.tx.QueryRowContext(ctx, stmt, volumeID).Scan(&response.ID, &response.Name, &response.Description, &response.CreationDate, &response.Type, &response.NodeID, &response.PoolName, &response.ProjectName)
 	if err != nil {
-		if err == sql.ErrNoRows {
+		if errors.Is(err, sql.ErrNoRows) {
 			return StorageVolumeArgs{}, api.StatusErrorf(http.StatusNotFound, "Storage pool volume not found")
 		}
 
@@ -160,11 +160,11 @@ func (c *ClusterTx) GetStoragePoolVolumes(ctx context.Context, poolID int64, mem
 		for i, filter := range filters {
 			// Validate filter.
 			if filter.Name != nil && filter.Type == nil {
-				return nil, fmt.Errorf("Cannot filter by volume name if volume type not specified")
+				return nil, errors.New("Cannot filter by volume name if volume type not specified")
 			}
 
 			if filter.Name != nil && filter.Project == nil {
-				return nil, fmt.Errorf("Cannot filter by volume name if volume project not specified")
+				return nil, errors.New("Cannot filter by volume name if volume project not specified")
 			}
 
 			var qFilters []string
@@ -185,7 +185,7 @@ func (c *ClusterTx) GetStoragePoolVolumes(ctx context.Context, poolID int64, mem
 			}
 
 			if qFilters == nil {
-				return nil, fmt.Errorf("Invalid storage volume filter")
+				return nil, errors.New("Invalid storage volume filter")
 			}
 
 			if i > 0 {
@@ -440,7 +440,7 @@ func (c *ClusterTx) CreateStoragePoolVolume(ctx context.Context, projectName str
 	var volumeID int64
 
 	if internalInstance.IsSnapshot(volumeName) {
-		return -1, fmt.Errorf("Volume name may not be a snapshot")
+		return -1, errors.New("Volume name may not be a snapshot")
 	}
 
 	remoteDrivers := StorageRemoteDriverNames()
@@ -649,7 +649,7 @@ func (c *ClusterTx) GetStorageVolumeNodes(ctx context.Context, poolID int64, pro
 		// equivalent to db.ErrNoClusterMember that is used in newer schemas where a single remote volume
 		// DB record is created that is not associated to any single member.
 		if StorageRemoteDriverNames == nil {
-			return nil, fmt.Errorf("No remote storage drivers function defined")
+			return nil, errors.New("No remote storage drivers function defined")
 		}
 
 		remoteDrivers := StorageRemoteDriverNames()
@@ -815,7 +815,7 @@ func StoragePoolVolumeTypeToName(volumeType int) (string, error) {
 		return StoragePoolVolumeTypeNameCustom, nil
 	}
 
-	return "", fmt.Errorf("Invalid storage volume type")
+	return "", errors.New("Invalid storage volume type")
 }
 
 // Convert a volume integer content type code to its human-readable name.
@@ -829,7 +829,7 @@ func storagePoolVolumeContentTypeToName(contentType int) (string, error) {
 		return StoragePoolVolumeContentTypeNameISO, nil
 	}
 
-	return "", fmt.Errorf("Invalid storage volume content type")
+	return "", errors.New("Invalid storage volume content type")
 }
 
 // GetCustomVolumesInProject returns all custom volumes in the given project.
