@@ -1,8 +1,10 @@
 package drivers
 
 import (
+	"errors"
 	"fmt"
 	"io"
+	"maps"
 	"net/url"
 	"os/exec"
 	"regexp"
@@ -55,9 +57,7 @@ func (d *common) validatePool(config map[string]string, driverRules map[string]f
 	rules := d.commonRules.PoolRules()
 
 	// Merge driver specific rules into common rules.
-	for field, validator := range driverRules {
-		rules[field] = validator
-	}
+	maps.Copy(rules, driverRules)
 
 	// Add to pool volume configuration options as volume.* options.
 	// These will be used as default configuration options for volume.
@@ -104,13 +104,7 @@ func (d *common) fillVolumeConfig(vol *Volume, excludedKeys ...string) error {
 
 		volKey := strings.TrimPrefix(k, "volume.")
 
-		isExcluded := false
-		for _, excludedKey := range excludedKeys {
-			if excludedKey == volKey {
-				isExcluded = true
-				break
-			}
-		}
+		isExcluded := slices.Contains(excludedKeys, volKey)
 
 		if isExcluded {
 			continue
@@ -155,9 +149,7 @@ func (d *common) validateVolume(vol Volume, driverRules map[string]func(value st
 	rules := d.commonRules.VolumeRules(vol)
 
 	// Merge driver specific rules into common rules.
-	for field, validator := range driverRules {
-		rules[field] = validator
-	}
+	maps.Copy(rules, driverRules)
 
 	// Run the validator against each field.
 	for k, validator := range rules {
@@ -194,7 +186,7 @@ func (d *common) validateVolume(vol Volume, driverRules map[string]func(value st
 
 	// Check that security.unmapped and security.shifted are not set together.
 	if util.IsTrue(vol.config["security.unmapped"]) && util.IsTrue(vol.config["security.shifted"]) {
-		return fmt.Errorf("security.unmapped and security.shifted are mutually exclusive")
+		return errors.New("security.unmapped and security.shifted are mutually exclusive")
 	}
 
 	return nil
@@ -303,10 +295,10 @@ func (d *common) moveGPTAltHeader(devPath string) error {
 		return nil
 	}
 
-	runErr, ok := err.(subprocess.RunError)
-	if ok {
-		exitError, ok := runErr.Unwrap().(*exec.ExitError)
-		if ok {
+	var runErr subprocess.RunError
+	if errors.As(err, &runErr) {
+		var exitError *exec.ExitError
+		if errors.As(runErr.Unwrap(), &exitError) {
 			// sgdisk manpage says exit status 3 means:
 			// "Non-GPT disk detected and no -g option, but operation requires a write action".
 			if exitError.ExitCode() == 3 {
@@ -318,23 +310,6 @@ func (d *common) moveGPTAltHeader(devPath string) error {
 	return err
 }
 
-// runFiller runs the supplied filler, and setting the returned volume size back into filler.
-func (d *common) runFiller(vol Volume, devPath string, filler *VolumeFiller, allowUnsafeResize bool) error {
-	if filler == nil || filler.Fill == nil {
-		return nil
-	}
-
-	vol.driver.Logger().Debug("Running filler function", logger.Ctx{"dev": devPath, "path": vol.MountPath()})
-	volSize, err := filler.Fill(vol, devPath, allowUnsafeResize)
-	if err != nil {
-		return err
-	}
-
-	filler.Size = volSize
-
-	return nil
-}
-
 // CreateVolume creates a new storage volume on disk.
 func (d *common) CreateVolume(vol Volume, filler *VolumeFiller, op *operations.Operation) error {
 	return ErrNotSupported
@@ -432,7 +407,7 @@ func (d *common) MigrateVolume(vol Volume, conn io.ReadWriteCloser, volSrcArgs *
 }
 
 // BackupVolume creates an exported version of a volume.
-func (d *common) BackupVolume(vol Volume, tarWriter *instancewriter.InstanceTarWriter, optimized bool, snapshots []string, op *operations.Operation) error {
+func (d *common) BackupVolume(vol Volume, writer instancewriter.InstanceWriter, optimized bool, snapshots []string, op *operations.Operation) error {
 	return ErrNotSupported
 }
 
@@ -475,7 +450,7 @@ func (d *common) RenameVolumeSnapshot(snapVol Volume, newSnapshotName string, op
 func (d *common) ValidateBucket(bucket Volume) error {
 	projectName, bucketName := project.StorageVolumeParts(bucket.name)
 	if projectName == "" {
-		return fmt.Errorf("Project prefix missing in bucket volume name")
+		return errors.New("Project prefix missing in bucket volume name")
 	}
 
 	match, err := regexp.MatchString(`^[a-z0-9][\-\.a-z0-9]{2,62}$`, bucketName)
@@ -484,7 +459,7 @@ func (d *common) ValidateBucket(bucket Volume) error {
 	}
 
 	if !match {
-		return fmt.Errorf("Bucket name must be between 3 and 63 lowercase letters, numbers, periods or hyphens and must start with a letter or number")
+		return errors.New("Bucket name must be between 3 and 63 lowercase letters, numbers, periods or hyphens and must start with a letter or number")
 	}
 
 	return nil
@@ -513,12 +488,12 @@ func (d *common) UpdateBucket(bucket Volume, changedConfig map[string]string) er
 // ValidateBucketKey validates the supplied bucket key config.
 func (d *common) ValidateBucketKey(keyName string, creds S3Credentials, roleName string) error {
 	if keyName == "" {
-		return fmt.Errorf("Key name is required")
+		return errors.New("Key name is required")
 	}
 
 	validRoles := []string{"admin", "read-only"}
 	if !slices.Contains(validRoles, roleName) {
-		return fmt.Errorf("Invalid key role")
+		return errors.New("Invalid key role")
 	}
 
 	return nil
@@ -577,3 +552,18 @@ func (d *common) filesystemFreeze(path string) (func() error, error) {
 
 	return unfreezeFS, nil
 }
+
+// CacheVolumeSnapshots causes snapshot data to be cached for later use (for bulk queries).
+func (d *common) CacheVolumeSnapshots(vol Volume) error {
+	return nil
+}
+
+// GetQcow2BackingFilePath generates the backing file path for the specified volume.
+func (d *common) GetQcow2BackingFilePath(vol Volume) (string, error) {
+	return "", ErrNotSupported
+}
+
+// Qcow2DeletionCleanup performs post block-commit cleanup of qcow2 snapshot artifacts.
+func (d *common) Qcow2DeletionCleanup(vol Volume, childName string) error {
+	return ErrNotSupported
+}
