@@ -13,7 +13,9 @@ var drivers = map[string]func() driver{
 	"dir":        func() driver { return &dir{} },
 	"lvm":        func() driver { return &lvm{} },
 	"lvmcluster": func() driver { return &lvm{clustered: true} },
+	"truenas":    func() driver { return &truenas{} },
 	"zfs":        func() driver { return &zfs{} },
+	"linstor":    func() driver { return &linstor{} },
 }
 
 // Validators contains functions used for validating a drivers's config.
