@@ -5,6 +5,7 @@ import (
 	"encoding/json"
 	"errors"
 	"fmt"
+	"io"
 	"io/fs"
 	"os"
 	"path/filepath"
@@ -89,7 +90,7 @@ func VolumeTypeNameToDBType(volumeTypeName string) (int, error) {
 		return db.StoragePoolVolumeTypeCustom, nil
 	}
 
-	return -1, fmt.Errorf("Invalid storage volume type name")
+	return -1, errors.New("Invalid storage volume type name")
 }
 
 // VolumeTypeToDBType converts volume type to internal volume type DB code.
@@ -133,7 +134,7 @@ func InstanceTypeToVolumeType(instType instancetype.Type) (drivers.VolumeType, e
 		return drivers.VolumeTypeVM, nil
 	}
 
-	return "", fmt.Errorf("Invalid instance type")
+	return "", errors.New("Invalid instance type")
 }
 
 // VolumeTypeToAPIInstanceType converts storage driver volume type to API instance type type.
@@ -145,7 +146,7 @@ func VolumeTypeToAPIInstanceType(volType drivers.VolumeType) (api.InstanceType,
 		return api.InstanceTypeVM, nil
 	}
 
-	return api.InstanceTypeAny, fmt.Errorf("Volume type doesn't have equivalent instance type")
+	return api.InstanceTypeAny, errors.New("Volume type doesn't have equivalent instance type")
 }
 
 // VolumeContentTypeToDBContentType converts volume type to internal code.
@@ -159,7 +160,7 @@ func VolumeContentTypeToDBContentType(contentType drivers.ContentType) (int, err
 		return db.StoragePoolVolumeContentTypeISO, nil
 	}
 
-	return -1, fmt.Errorf("Invalid volume content type")
+	return -1, errors.New("Invalid volume content type")
 }
 
 // VolumeDBContentTypeToContentType converts internal content type DB code to driver representation.
@@ -173,7 +174,7 @@ func VolumeDBContentTypeToContentType(volDBType int) (drivers.ContentType, error
 		return drivers.ContentTypeISO, nil
 	}
 
-	return "", fmt.Errorf("Invalid volume content type")
+	return "", errors.New("Invalid volume content type")
 }
 
 // VolumeContentTypeNameToContentType converts volume content type string internal code.
@@ -187,14 +188,14 @@ func VolumeContentTypeNameToContentType(contentTypeName string) (int, error) {
 		return db.StoragePoolVolumeContentTypeISO, nil
 	}
 
-	return -1, fmt.Errorf("Invalid volume content type name")
+	return -1, errors.New("Invalid volume content type name")
 }
 
 // VolumeDBGet loads a volume from the database.
 func VolumeDBGet(pool Pool, projectName string, volumeName string, volumeType drivers.VolumeType) (*db.StorageVolume, error) {
 	p, ok := pool.(*backend)
 	if !ok {
-		return nil, fmt.Errorf("Pool is not a backend")
+		return nil, errors.New("Pool is not a backend")
 	}
 
 	volDBType, err := VolumeTypeToDBType(volumeType)
@@ -229,12 +230,12 @@ func VolumeDBGet(pool Pool, projectName string, volumeName string, volumeType dr
 func VolumeDBCreate(pool Pool, projectName string, volumeName string, volumeDescription string, volumeType drivers.VolumeType, snapshot bool, volumeConfig map[string]string, creationDate time.Time, expiryDate time.Time, contentType drivers.ContentType, removeUnknownKeys bool, hasSource bool) error {
 	p, ok := pool.(*backend)
 	if !ok {
-		return fmt.Errorf("Pool is not a backend")
+		return errors.New("Pool is not a backend")
 	}
 
 	// Prevent using this function to create storage volume bucket records.
 	if volumeType == drivers.VolumeTypeBucket {
-		return fmt.Errorf("Cannot store volume using bucket type")
+		return errors.New("Cannot store volume using bucket type")
 	}
 
 	// If the volumeType represents an instance type then check that the volumeConfig doesn't contain any of
@@ -274,10 +275,12 @@ func VolumeDBCreate(pool Pool, projectName string, volumeName string, volumeDesc
 	// Set source indicator.
 	vol.SetHasSource(hasSource)
 
-	// Fill default config.
-	err = pool.Driver().FillVolumeConfig(vol)
-	if err != nil {
-		return err
+	// For new volumes, fill default config.
+	if !snapshot {
+		err = pool.Driver().FillVolumeConfig(vol)
+		if err != nil {
+			return err
+		}
 	}
 
 	// Validate config.
@@ -307,7 +310,7 @@ func VolumeDBCreate(pool Pool, projectName string, volumeName string, volumeDesc
 func VolumeDBDelete(pool Pool, projectName string, volumeName string, volumeType drivers.VolumeType) error {
 	p, ok := pool.(*backend)
 	if !ok {
-		return fmt.Errorf("Pool is not a backend")
+		return errors.New("Pool is not a backend")
 	}
 
 	// Convert the volume type to our internal integer representation.
@@ -330,7 +333,7 @@ func VolumeDBDelete(pool Pool, projectName string, volumeName string, volumeType
 func VolumeDBSnapshotsGet(pool Pool, projectName string, volume string, volumeType drivers.VolumeType) ([]db.StorageVolumeArgs, error) {
 	p, ok := pool.(*backend)
 	if !ok {
-		return nil, fmt.Errorf("Pool is not a backend")
+		return nil, errors.New("Pool is not a backend")
 	}
 
 	volDBType, err := VolumeTypeToDBType(volumeType)
@@ -356,7 +359,7 @@ func VolumeDBSnapshotsGet(pool Pool, projectName string, volume string, volumeTy
 func BucketDBGet(pool Pool, projectName string, bucketName string, memberSpecific bool) (*db.StorageBucket, error) {
 	p, ok := pool.(*backend)
 	if !ok {
-		return nil, fmt.Errorf("Pool is not a backend")
+		return nil, errors.New("Pool is not a backend")
 	}
 
 	var err error
@@ -388,7 +391,7 @@ func BucketDBGet(pool Pool, projectName string, bucketName string, memberSpecifi
 func BucketDBCreate(ctx context.Context, pool Pool, projectName string, memberSpecific bool, bucket *api.StorageBucketsPost) (int64, error) {
 	p, ok := pool.(*backend)
 	if !ok {
-		return -1, fmt.Errorf("Pool is not a backend")
+		return -1, errors.New("Pool is not a backend")
 	}
 
 	// Make sure that we don't pass a nil to the next function.
@@ -436,7 +439,7 @@ func BucketDBCreate(ctx context.Context, pool Pool, projectName string, memberSp
 func BucketDBDelete(ctx context.Context, pool Pool, bucketID int64) error {
 	p, ok := pool.(*backend)
 	if !ok {
-		return fmt.Errorf("Pool is not a backend")
+		return errors.New("Pool is not a backend")
 	}
 
 	err := p.state.DB.Cluster.Transaction(ctx, func(ctx context.Context, tx *db.ClusterTx) error {
@@ -453,7 +456,7 @@ func BucketDBDelete(ctx context.Context, pool Pool, bucketID int64) error {
 func BucketKeysDBGet(pool Pool, bucketID int64) ([]*db.StorageBucketKey, error) {
 	p, ok := pool.(*backend)
 	if !ok {
-		return nil, fmt.Errorf("Pool is not a backend")
+		return nil, errors.New("Pool is not a backend")
 	}
 
 	var err error
@@ -487,6 +490,11 @@ func poolAndVolumeCommonRules(vol *drivers.Volume) map[string]func(string) error
 			_, err := internalInstance.GetExpiry(time.Time{}, value)
 			return err
 		},
+		"snapshots.expiry.manual": func(value string) error {
+			// Validate expression
+			_, err := internalInstance.GetExpiry(time.Time{}, value)
+			return err
+		},
 		"snapshots.schedule": validate.Optional(validate.IsCron([]string{"@hourly", "@daily", "@midnight", "@weekly", "@monthly", "@annually", "@yearly"})),
 		"snapshots.pattern":  validate.IsAny,
 	}
@@ -567,7 +575,7 @@ func validateVolumeCommonRules(vol drivers.Volume) map[string]func(string) error
 // VM Format A: Separate metadata tarball and root qcow2 file.
 //   - Unpack metadata tarball into mountPath.
 //   - Check rootBlockPath is a file and convert qcow2 file into raw format in rootBlockPath.
-func ImageUnpack(imageFile string, vol drivers.Volume, destBlockFile string, sysOS *sys.OS, allowUnsafeResize bool, tracker *ioprogress.ProgressTracker) (int64, error) {
+func ImageUnpack(imageFile string, vol drivers.Volume, destBlockFile string, sysOS *sys.OS, allowUnsafeResize bool, targetIsZero bool, tracker *ioprogress.ProgressTracker, targetFormat string) (int64, error) {
 	l := logger.Log.AddContext(logger.Ctx{"imageFile": imageFile, "volName": vol.Name()})
 	l.Info("Image unpack started")
 	defer l.Info("Image unpack stopped")
@@ -599,7 +607,7 @@ func ImageUnpack(imageFile string, vol drivers.Volume, destBlockFile string, sys
 		if util.PathExists(imageRootfsFile) {
 			err = os.MkdirAll(rootfsPath, 0o755)
 			if err != nil {
-				return -1, fmt.Errorf("Error creating rootfs directory")
+				return -1, errors.New("Error creating rootfs directory")
 			}
 
 			err = archive.Unpack(imageRootfsFile, rootfsPath, vol.IsBlockBacked(), maxMemory, tracker)
@@ -655,7 +663,7 @@ func ImageUnpack(imageFile string, vol drivers.Volume, destBlockFile string, sys
 		}
 
 		// Belt and braces qcow2 check.
-		if imgInfo.Format != "qcow2" {
+		if imgInfo.Format != drivers.BlockVolumeTypeQcow2 {
 			return -1, fmt.Errorf("Unexpected image format %q", imgInfo.Format)
 		}
 
@@ -690,6 +698,44 @@ func ImageUnpack(imageFile string, vol drivers.Volume, destBlockFile string, sys
 			}
 		}
 
+		if targetFormat == drivers.BlockVolumeTypeQcow2 {
+			l.Debug("Writing qcow2 image to disk", logger.Ctx{"imgPath": imgPath, "dstPath": dstPath})
+
+			// Attempt to deref all paths.
+			imgFullPath, err := filepath.EvalSymlinks(imgPath)
+			if err == nil {
+				imgPath = imgFullPath
+			}
+
+			if dstPath != "" {
+				dstFullPath, err := filepath.EvalSymlinks(dstPath)
+				if err == nil {
+					dstPath = dstFullPath
+				}
+			}
+
+			from, err := os.OpenFile(imgPath, unix.O_RDONLY, 0)
+			if err != nil {
+				return -1, err
+			}
+
+			defer from.Close()
+
+			to, err := os.OpenFile(dstPath, unix.O_WRONLY|os.O_CREATE|os.O_TRUNC, 0)
+			if err != nil {
+				return -1, err
+			}
+
+			defer to.Close()
+
+			_, err = io.Copy(to, from)
+			if err != nil {
+				return -1, err
+			}
+
+			return imgInfo.VirtualSize, nil
+		}
+
 		// Convert the qcow2 format to a raw block device.
 		l.Debug("Converting qcow2 image to raw disk", logger.Ctx{"imgPath": imgPath, "dstPath": dstPath})
 
@@ -716,8 +762,10 @@ func ImageUnpack(imageFile string, vol drivers.Volume, destBlockFile string, sys
 			// Parallel unpacking.
 			cmd = append(cmd, "-W")
 
-			// Our block devices are clean, so skip zeroes.
-			cmd = append(cmd, "-n", "--target-is-zero")
+			if targetIsZero {
+				// Our block devices are clean, so skip zeroes.
+				cmd = append(cmd, "-n", "--target-is-zero")
+			}
 		}
 
 		cmd = append(cmd, imgPath, dstPath)
@@ -785,7 +833,7 @@ func ImageUnpack(imageFile string, vol drivers.Volume, destBlockFile string, sys
 }
 
 // InstanceContentType returns the instance's content type.
-func InstanceContentType(inst instance.Instance) drivers.ContentType {
+func InstanceContentType(inst instance.ConfigReader) drivers.ContentType {
 	contentType := drivers.ContentTypeFS
 	if inst.Type() == instancetype.VM {
 		contentType = drivers.ContentTypeBlock
@@ -1002,7 +1050,7 @@ func VolumeUsedByExclusiveRemoteInstancesWithProfiles(s *state.State, poolName s
 
 		return nil
 	})
-	if err != nil && err != db.ErrInstanceListStop {
+	if err != nil && !errors.Is(err, db.ErrInstanceListStop) {
 		return nil, err
 	}
 
@@ -1045,30 +1093,6 @@ func FallbackMigrationType(contentType drivers.ContentType) migration.MigrationF
 	return migration.MigrationFSType_RSYNC
 }
 
-// RenderSnapshotUsage can be used as an optional argument to Instance.Render() to return snapshot usage.
-// As this is a relatively expensive operation it is provided as an optional feature rather than on by default.
-func RenderSnapshotUsage(s *state.State, snapInst instance.Instance) func(response any) error {
-	return func(response any) error {
-		apiRes, ok := response.(*api.InstanceSnapshot)
-		if !ok {
-			return nil
-		}
-
-		pool, err := LoadByInstance(s, snapInst)
-		if err == nil {
-			// It is important that the snapshot not be mounted here as mounting a snapshot can trigger a very
-			// expensive filesystem UUID regeneration, so we rely on the driver implementation to get the info
-			// we are requesting as cheaply as possible.
-			volumeState, err := pool.GetInstanceUsage(snapInst)
-			if err == nil {
-				apiRes.Size = volumeState.Used
-			}
-		}
-
-		return nil
-	}
-}
-
 // InstanceMount mounts an instance's storage volume (if not already mounted).
 // Please call InstanceUnmount when finished.
 func InstanceMount(pool Pool, inst instance.Instance, op *operations.Operation) (*MountInfo, error) {
@@ -1114,7 +1138,7 @@ func InstanceDiskBlockSize(pool Pool, inst instance.Instance, op *operations.Ope
 	defer func() { _ = InstanceUnmount(pool, inst, op) }()
 
 	if mountInfo.DiskPath == "" {
-		return -1, fmt.Errorf("No disk path available from mount")
+		return -1, errors.New("No disk path available from mount")
 	}
 
 	blockDiskSize, err := drivers.BlockDiskSizeBytes(mountInfo.DiskPath)
@@ -1205,7 +1229,7 @@ func CalculateVolumeSnapshotSize(projectName string, pool Pool, contentType driv
 	err := snapVol.MountTask(func(mountPath string, op *operations.Operation) error {
 		poolBackend, ok := pool.(*backend)
 		if !ok {
-			return fmt.Errorf("Pool is not a backend")
+			return errors.New("Pool is not a backend")
 		}
 
 		volDiskPath, err := poolBackend.driver.GetVolumeDiskPath(snapVol)
