@@ -17,7 +17,8 @@ import (
 // can not be found (404 HTTP status code).
 var ErrNotFound = errors.New("resource not found")
 
-func DownloadFileHash(ctx context.Context, httpClient *http.Client, useragent string, progress func(progress ioprogress.ProgressData), canceler *cancel.HTTPRequestCanceller, filename string, url string, hash string, hashFunc hash.Hash, target io.WriteSeeker) (int64, error) {
+// DownloadFileHash downloads a file while validating its hash.
+func DownloadFileHash(ctx context.Context, httpClient *http.Client, useragent string, progress func(progress ioprogress.ProgressData), canceler *cancel.HTTPRequestCanceller, filename string, url string, fileHash string, hashFunc hash.Hash, target io.WriteSeeker) (int64, error) {
 	// Always seek to the beginning
 	_, _ = target.Seek(0, io.SeekStart)
 
@@ -83,8 +84,8 @@ func DownloadFileHash(ctx context.Context, httpClient *http.Client, useragent st
 		}
 
 		result := fmt.Sprintf("%x", hashFunc.Sum(nil))
-		if result != hash {
-			return -1, fmt.Errorf("Hash mismatch for %s: %s != %s", url, result, hash)
+		if result != fileHash {
+			return -1, fmt.Errorf("Hash mismatch for %s: %s != %s", url, result, fileHash)
 		}
 	} else {
 		size, err = io.Copy(target, body)
