@@ -1,4 +1,7 @@
 test_migration() {
+    # shellcheck disable=2153
+    [ "${INCUS_BACKEND}" = "linstor" ] && echo "==> SKIP: Linstor can't currently handle copies with snapshots" && return
+
     # setup a second Incus
     # shellcheck disable=2039,3043
     local INCUS2_DIR INCUS2_ADDR incus_backend
@@ -57,11 +60,6 @@ test_migration() {
                 continue
             fi
 
-            if [ "${fs}" = "xfs" ] && grep -q noble /etc/os-release; then
-                echo "==> SKIP: Skipping block mode test on ${fs} due to broken OS tools."
-                continue
-            fi
-
             # shellcheck disable=2039,3043
             local storage_pool1 storage_pool2
             # shellcheck disable=2153
